@@ -1,6 +1,6 @@
 /*
 #   compressor.h: Audio dynamic range compression code from IDJC.
-#   Copyright (C) 2005 Stephen Fairchild
+#   Copyright (C) 2005-2006 Stephen Fairchild
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -19,6 +19,7 @@
 
 #include <jack/jack.h>
 #include "dsp.h"
+#include "rms_calc.h"
 
 typedef jack_default_audio_sample_t compaudio_t;
 
@@ -32,12 +33,14 @@ struct compressor
    compaudio_t release;
    compaudio_t opgain;
    compaudio_t ducking;
+   compaudio_t curve;
    int ducking_hold;
    int ducking_hold_count;
    compaudio_t ducking_db;
    compaudio_t de_ess_db;
    compaudio_t signal_db;
    struct digital_filter *filter;
+   struct rms_calc *rms_filter;
    };
 
 struct normalizer
@@ -50,6 +53,6 @@ struct normalizer
    compaudio_t maxlevel;
    };
 
-compaudio_t compressor(struct compressor *self, compaudio_t left, compaudio_t right);
+compaudio_t compressor(struct compressor *self, compaudio_t signal, int skip_rms);
 compaudio_t limiter(struct compressor *self, compaudio_t left, compaudio_t right);
 compaudio_t normalizer(struct normalizer *self, compaudio_t left, compaudio_t right);
