@@ -1,6 +1,6 @@
 /*
-#   idjcmixer.c: IDJC's mixer.
-#   Copyright (C) 2006 Stephen Fairchild
+#   idjcmixer.c: central core of IDJC's mixer.
+#   Copyright (C) 2005-2006 Stephen Fairchild
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -275,6 +275,9 @@ dfilter_t b_ngbandpass[] = { 7.0389284721, -21.9220056594, 39.5069302814, -45.10
 size_t b_ngbandpasscount = 8;
 dfilter_t ngbandpassgain = 6.381624743e+02;
 
+/* the rms filter currently in use */
+struct rms_calc *lm_rms_filter, *rm_rms_filter;
+
 /* Does this ever get run, or is it cruft? */
 void process_silence(jack_nframes_t nframes)
    {
@@ -648,11 +651,13 @@ void process_audio(jack_nframes_t nframes)
       new_mic_compressor.ducking_db = left_mic_compressor.ducking_db;
       left_mic_compressor = new_mic_compressor;
       left_mic_compressor.filter = ess_left_filter;
+      left_mic_compressor.rms_filter = lm_rms_filter;
       new_mic_compressor.gain_db = right_mic_compressor.gain_db;
       new_mic_compressor.ducking_hold_count = right_mic_compressor.gain_db;
       new_mic_compressor.ducking_db = right_mic_compressor.ducking_db;
       right_mic_compressor = new_mic_compressor;
       right_mic_compressor.filter = ess_right_filter;
+      right_mic_compressor.rms_filter = rm_rms_filter;
       new_compressor_stats = FALSE;
       }
    if (new_noisegate_stats)
@@ -674,7 +679,7 @@ void process_audio(jack_nframes_t nframes)
       str_normalizer = new_normalizer;
       new_normalizer_stats = FALSE;
       }
-      
+
    /* apply the user selected microphone filter */
    if (mic_filter == 1)
       {
@@ -712,14 +717,14 @@ void process_audio(jack_nframes_t nframes)
 	    }
          
 	 /* perform the sidechain actions of the compressor and noisegate */
-	 compressor_gain = compressor(&left_mic_compressor, lmic, 0.0F);
+	 compressor_gain = compressor(&left_mic_compressor, lmic, 0);
 	 noisegate_gain = noisegate(&left_mic_noisegate, digital_filter(ng_left_filter, lmic));
 	 /* apply the sidechain results to the input signal */
 	 
         lmic *= db2level(compressor_gain + noisegate_gain);
 	 
 	 /* do the same for the right microphone */	 
-	 compressor_gain = compressor(&right_mic_compressor, rmic, 0.0F);
+	 compressor_gain = compressor(&right_mic_compressor, rmic, 0);
 	 noisegate_gain = noisegate(&right_mic_noisegate, digital_filter(ng_right_filter, rmic));
 	 
 	 rmic *= db2level(compressor_gain + noisegate_gain);
@@ -900,14 +905,14 @@ void process_audio(jack_nframes_t nframes)
                }
             
             /* perform the sidechain actions of the compressor and noisegate */
-            compressor_gain = compressor(&left_mic_compressor, lmic, 0.0F);
+            compressor_gain = compressor(&left_mic_compressor, lmic, 0);
 	    noisegate_gain = noisegate(&left_mic_noisegate, digital_filter(ng_left_filter, lmic));            
             /* apply the sidechain results to the input signal */
             
             lmic *= db2level(compressor_gain + noisegate_gain);
             
             /* do the same for the right microphone */	 
-            compressor_gain = compressor(&right_mic_compressor, rmic, 0.0F);
+            compressor_gain = compressor(&right_mic_compressor, rmic, 0);
 	    noisegate_gain = noisegate(&right_mic_noisegate, digital_filter(ng_right_filter, rmic));            
             
             rmic *= db2level(compressor_gain + noisegate_gain);
@@ -1082,14 +1087,14 @@ void process_audio(jack_nframes_t nframes)
                   }
                
                /* perform the sidechain actions of the compressor and noisegate */
-               compressor_gain = compressor(&left_mic_compressor, lmic, 0.0F);
+               compressor_gain = compressor(&left_mic_compressor, lmic, 0);
  	       noisegate_gain = noisegate(&left_mic_noisegate, digital_filter(ng_left_filter, lmic));            
                /* apply the sidechain results to the input signal */
                
                lmic *= db2level(compressor_gain + noisegate_gain);
                
                /* do the same for the right microphone */	 
-               compressor_gain = compressor(&right_mic_compressor, rmic, 0.0F);
+               compressor_gain = compressor(&right_mic_compressor, rmic, 0);
  	       noisegate_gain = noisegate(&right_mic_noisegate, digital_filter(ng_right_filter, rmic));            
                
                rmic *= db2level(compressor_gain + noisegate_gain);
@@ -1262,14 +1267,14 @@ void process_audio(jack_nframes_t nframes)
                      }
                   
                   /* perform the sidechain actions of the compressor and noisegate */
-                  compressor_gain = compressor(&left_mic_compressor, lmic, 0.0F);
+                  compressor_gain = compressor(&left_mic_compressor, lmic, 0);
  	          noisegate_gain = noisegate(&left_mic_noisegate, digital_filter(ng_left_filter, lmic));            
                   
                   /* apply the sidechain results to the input signal */
                   lmic *= db2level(compressor_gain + noisegate_gain);
                   
                   /* do the same for the right microphone */	 
-                  compressor_gain = compressor(&right_mic_compressor, rmic, 0.0F);
+                  compressor_gain = compressor(&right_mic_compressor, rmic, 0);
  	          noisegate_gain = noisegate(&right_mic_noisegate, digital_filter(ng_right_filter, rmic));            
                   
                   rmic *= db2level(compressor_gain + noisegate_gain);
@@ -1518,7 +1523,7 @@ void askxine_segfault_handler(int data)
    {
    fprintf(stderr, "Segfaulted during attempted creation of the askxine pseudoplayer\n");
    fprintf(stderr, "This is usually the result of a corrupt config file in ~/.xine\n");
-   fprintf(stderr, "Recommend that you delete your xine config directory\n");
+   fprintf(stderr, "Recommend that you delete your ~/.xine directory\n");
    printf("Segmentation Fault\n");
    fflush(stdout);
    exit(5);
@@ -1528,8 +1533,6 @@ void regular_segfault_handler(int data)
    {
    printf("Segmentation Fault\n");
    fflush(stdout);
-   if (client)
-      jack_client_close(client);
    exit(5);
    }
 
@@ -1538,7 +1541,34 @@ void jack_shutdown_handler(void *data)
    jack_closed_f = TRUE;
    }
 
-int main()
+void alarm_handler(int data)
+   {
+   struct xlplayer *cause;
+   
+   if ((cause = plr_l)->watchdog_timer++ == 4 ||
+       (cause = plr_r)->watchdog_timer++ == 4 ||
+       (cause = plr_i)->watchdog_timer++ == 4 ||
+       (cause = plr_j)->watchdog_timer++ == 4) 
+      {
+      if (cause->playmode == XL_INITIATE)
+         {
+         cause->initial_audio_context = cause->current_audio_context;
+         cause->playmode = XL_STOPPED;
+         cause->command = XL_COMPLETE;
+         }
+      fprintf(stderr, "watchdog timer frozen for one of the media players -- possible bad media file\nshutting down the mixer in one second\n");
+      signal(SIGALRM, regular_segfault_handler);
+      }
+   alarm(1);
+   }
+
+void atexit_handler()
+   {
+   if (client)
+      jack_client_close(client);
+   }
+
+int main(int argc, char **argv)
    {
    FILE *fp=stdin;
    char *client_name = 0;
@@ -1549,6 +1579,7 @@ int main()
    float normrise, normfall;
    float comp_ratio;
    int comp_knee, comp_depth, comp_attack, comp_release, comp_gain, comp_relative_f, comp_ducking_hold, fadeout_f;
+   int filterslow;
    int flush_left, flush_right, flush_jingles, flush_interlude;
    float comp_ducking, de_ess;
    int mic_left_invert, mic_right_invert, mic_left_level, mic_right_level,
@@ -1560,13 +1591,15 @@ int main()
    char *artist = NULL, *title = NULL;
    double length;
    int sync = FALSE;
-    
+   struct rms_calc *lm_fast_rms, *lm_slow_rms, *rm_fast_rms, *rm_slow_rms;
+
    if (sigfillset(&mixersigset))
       {
       fprintf(stderr, "sigfillset failed\n");
       return 1;
       }
    signal(SIGINT, interrupt_handler);
+   signal(SIGALRM, alarm_handler); 
    
    /* the following can cause a segfault when the xine config file is corrupted in certain ways */
    signal(SIGSEGV, askxine_segfault_handler);
@@ -1581,6 +1614,7 @@ int main()
    client_name = (char *) malloc(9 * sizeof (char));
    strcpy(client_name, "idjc-mx");
 
+   atexit(atexit_handler);
    if((client = jack_client_new(client_name)) == 0)
       {
       printf("IDJC: Error\n");
@@ -1630,7 +1664,7 @@ int main()
       printf("failed to create the fadein lookup table for xlplayer\n");
       return 1;
       }
-      
+
    if(! ((plr_l = xlplayer_create(sr, RB_SIZE, "leftplayer", &jack_closed_f)) &&
          (plr_r = xlplayer_create(sr, RB_SIZE, "rightplayer", &jack_closed_f))))
       {
@@ -1657,6 +1691,13 @@ int main()
    mild_left_mic_filter = new_digital_filter(a_butterworth90_count, a_butterworth90, b_butterworth90_count, b_butterworth90, butterworth90_filter_gain);
    mild_right_mic_filter = new_digital_filter(a_butterworth90_count, a_butterworth90, b_butterworth90_count, b_butterworth90, butterworth90_filter_gain);
    
+   lm_fast_rms = rms_calc_create(sr, 3.0F   / 1000.0F);	/* used for RMS calculation in compressor */
+   rm_fast_rms = rms_calc_create(sr, 3.0F   / 1000.0F);
+   lm_slow_rms = rms_calc_create(sr, 30.0F  / 1000.0F);
+   rm_slow_rms = rms_calc_create(sr, 30.0F  / 1000.0F);
+   left_mic_compressor.rms_filter = lm_rms_filter = lm_fast_rms;
+   right_mic_compressor.rms_filter = rm_rms_filter = rm_fast_rms;
+
    left_mic_filter = mild_left_mic_filter;
    right_mic_filter = mild_right_mic_filter;
    
@@ -1754,11 +1795,11 @@ int main()
    /* the main app waits on this signal in order to prevent a race with the server code */
    fprintf(stdout, "IDJC: Sample rate %d\n", (int)sr);
    fflush(stdout);
-                    
+ 
    		/* Scan for physical audio IO ports to use as defaults */
    inport = jack_get_ports(client, NULL, NULL, JackPortIsPhysical | JackPortIsOutput);
    outport = jack_get_ports(client, NULL, NULL, JackPortIsPhysical | JackPortIsInput);
-   
+ 
    /* Make voip input audio available on the voip output so you can hear your own voice */
    /* and hear jingles and mixed back audio too */
    /* If the voip app provides this then you can break the connection in the voip app */
@@ -1766,6 +1807,8 @@ int main()
    jack_connect(client, "idjc-mx:voip_send_lt", "idjc-mx:voip_recv_lt");
    jack_connect(client, "idjc-mx:voip_send_rt", "idjc-mx:voip_recv_rt");
       
+   alarm(3);		/* handles timeouts on media player worker threads */
+      
    while (kvp_parse(kvpdict, fp))
       {
       if (jack_closed_f == TRUE)
@@ -1897,11 +1940,21 @@ int main()
                
       if (!strcmp(action, "compstats"))
          {
-	 if(sscanf(compressor_string,":%f:%d:%d:%d:%d:%d:%d:%f:%d:%f:", &comp_ratio, &comp_knee, &comp_depth, &comp_attack, &comp_release, &comp_gain, &comp_relative_f, &comp_ducking, &comp_ducking_hold, &de_ess) != 10)
+	 if(sscanf(compressor_string,":%f:%d:%d:%d:%d:%d:%d:%f:%d:%f:%d:", &comp_ratio, &comp_knee, &comp_depth, &comp_attack, &comp_release, &comp_gain, &comp_relative_f, &comp_ducking, &comp_ducking_hold, &de_ess, &filterslow) != 11)
 	    {
 	    fprintf(stderr, "mixer got bad compressor string\n");
 	    break;
 	    }
+         if (filterslow)
+            {
+            lm_rms_filter = lm_slow_rms;
+            rm_rms_filter = rm_slow_rms;
+            }
+         else
+            {
+            lm_rms_filter = lm_fast_rms;
+            rm_rms_filter = rm_fast_rms;
+            }
 	 new_mic_compressor.gain_db = 0.0;
 	 new_mic_compressor.k1 = comp_knee;
 	 new_mic_compressor.k2 = comp_knee - comp_depth;
@@ -1914,7 +1967,7 @@ int main()
 	 if (comp_relative_f)
 	    {
 	    new_mic_compressor.opgain = 0.0;
-	    new_mic_compressor.opgain = (-compressor(&new_mic_compressor, 1.0, 1.0)) + comp_gain;
+	    new_mic_compressor.opgain = (-compressor(&new_mic_compressor, 1.0, 1)) + comp_gain;
 	    }
 	 else
 	    new_mic_compressor.opgain = comp_gain;
@@ -2133,8 +2186,8 @@ int main()
             		 plr_l->have_data_f | (plr_l->current_audio_context & 0x1),
             		 plr_r->have_data_f | (plr_r->current_audio_context & 0x1),
             		 plr_i->have_data_f | (plr_i->current_audio_context & 0x1),
-            		 left_peak > 0.0003F || left_peak < 0.0F,
-            		 right_peak > 0.0003F || right_peak < 0.0F,
+            		 left_peak > 0.0008F || left_peak < 0.0F || plr_l->pause,
+            		 right_peak > 0.0008F || right_peak < 0.0F || plr_r->pause,
                          plr_l->current_audio_context,
                          plr_r->current_audio_context,
                          plr_j->current_audio_context,
@@ -2147,6 +2200,11 @@ int main()
          }
       }
    jack_client_close(client);
+   client = NULL;
+   rms_calc_destroy(lm_fast_rms);
+   rms_calc_destroy(lm_slow_rms);
+   rms_calc_destroy(rm_fast_rms);
+   rms_calc_destroy(rm_slow_rms);
    /*beat_free(beat_lp);
    beat_free(beat_rp);*/
    ifree(lp_lc);
