@@ -1,6 +1,6 @@
 /*
 #   idjcserver.c: IDJC code that communicates with the radio server 
-#   Copyright (C) 2005 Stephen Fairchild
+#   Copyright (C) 2005-2006 Stephen Fairchild
 #
 #   This program is free software; you can redistribute it and/or modify
 #   it under the terms of the GNU General Public License as published by
@@ -90,8 +90,7 @@ jack_ringbuffer_t *leftring, *rightring, *leftsring, *rightsring;
 int ringbuffer_can_write = FALSE, sringbuffer_can_write = FALSE, ringbuffer_flush = FALSE, sringbuffer_flush = FALSE, disconnect_f = FALSE, is_saving = FALSE;
 int keep_connection, keep_saving, keep_watchdog, newmeta;
 /* data passed from the GUI pointed to by these variables */
-char *server_type, *host, *port, *password, *name, *metatext, *format, *bandwidth, *action, *streamname, *streamurl, *streamdesc, *streamgenre, *mount, *status_tag, *save_file,  *save_artist, *save_title, *save_format, *save_bitrate, *public_s, *meta_s, *save_pause, *auto_pause, *ap_actuation_delay, *ap_cancel_thresh_log, *meta_artist, *meta_title, *abr_s,
-*stereo_s;
+char *server_type, *host, *port, *password, *name, *metatext, *format, *bandwidth, *action, *streamname, *streamurl, *streamdesc, *streamgenre, *mount, *status_tag, *save_file,  *save_artist, *save_title, *save_format, *save_bitrate, *public_s, *meta_s, *save_pause, *auto_pause, *ap_actuation_delay, *ap_cancel_thresh_log, *meta_artist, *meta_title, *abr_s, *stereo_s, *high_quality;
 int public_f, meta_f, pause_f, paused, abr_f, stereo_f;
 /* autopause stuff */
 int auto_pause_f, auto_pause_actuate_f, ap_actuation_delay_samples;
@@ -139,6 +138,7 @@ struct kvpdict kvpdict[] = {
          { "MTIT", &meta_title },
          { "ABRF", &abr_s },
          { "STER", &stereo_s },
+         { "QUAL", &high_quality },
          { "META", &metatext, &pm},	/* use a mutex lock on this one */
          { "", NULL }};
 
@@ -158,7 +158,7 @@ void process_silence(jack_nframes_t nframes)
    jack_ringbuffer_write(leftsring, (char *)buffer, todo * sizeof (sample_t));
    jack_ringbuffer_write(rightsring, (char *)buffer, todo * sizeof (sample_t));
    free(buffer);
-   } 
+   }
 
 void process_audio(jack_nframes_t nframes)
    {
@@ -204,20 +204,20 @@ void process_audio(jack_nframes_t nframes)
       jack_ringbuffer_reset(rightsring);
       sringbuffer_flush = FALSE;
       }
-   
+
    if (sringbuffer_can_write == TRUE)
       {
       jack_ringbuffer_write(leftsring, (char *)ls_buffer, stodo * sizeof (sample_t));
       jack_ringbuffer_write(rightsring, (char *)rs_buffer, stodo * sizeof (sample_t));
       }
    }
-   
+
 int process(jack_nframes_t nframes, void *arg)
    {
    if(transport_aware)
       {
       jack_position_t pos;
-      
+
       if(jack_transport_query(client, &pos) != JackTransportRolling)
          {
 	 process_silence(nframes);
@@ -227,7 +227,7 @@ int process(jack_nframes_t nframes, void *arg)
    process_audio(nframes);
    return 0;
    }
-   
+
 int sample_rate_change()
    {
    fprintf(stderr, "The sample rate changed!  Exiting...\n");
@@ -250,6 +250,7 @@ void *save_codec_feeder(void *args)
    enum crosstype { ZC_NORMAL, ZC_CLOSING, ZC_PAUSED, ZC_OPENING };
    enum crosstype zerocross = ZC_NORMAL;
    
+   usleep(20000);
    if(!(lbuff = malloc(sizeof (sample_t) * SRC_SIZE)))
       {
       fprintf(stderr, "Unable to allocate left buffer in save_codec_feeder\n");
@@ -443,6 +444,7 @@ void *server_thread(void *args)
    long qty_read, ret;
    shout_metadata_t *meta;
 
+   usleep(20000);
    shout_init();
    if (!(mp3buff = (unsigned char *)malloc(MP3_BUF_SIZ)))
       {
@@ -563,6 +565,7 @@ void *server_thread(void *args)
       {
       fprintf(stderr, "Connected to server...\n");
       total = 0;
+      usleep(20000);
       while (keep_connection == TRUE)
          {
          qty_read = read(reader[0], mp3buff, MP3_BUF_SIZ);
@@ -589,7 +592,6 @@ void *server_thread(void *args)
                newmeta = FALSE;
                pthread_mutex_unlock(&pm);
                shout_set_metadata(shout, meta);
-               fprintf(stderr, "metadata sent: %s\n", metatext);
                }
 	    else
                newmeta = FALSE;
@@ -624,6 +626,7 @@ void *codec_feeder(void *args)
    int pcmvalue;
    int i;
    
+   usleep(20000);
    if(!(lbuff = malloc(sizeof (sample_t) * SRC_SIZE)))
       {
       fprintf(stderr, "Unable to allocate left buffer in codec_feeder\n");
@@ -712,6 +715,7 @@ void *codec_streamer_ogg(void *args)
    sample_t *left_dm_buffer=NULL, *right_dm_buffer=NULL, *ldmptr, *rdmptr;
    float **buffer, *bptr;
       
+   usleep(20000);
    /* Shoutcast server initialisation */
    shout_init();
    if (!(shout = shout_new()))
@@ -835,6 +839,7 @@ void *codec_streamer_ogg(void *args)
       fprintf(stderr, "Connected to server...\n");
       ringbuffer_can_write = TRUE;		/* Turn on the flow of data from JACK */
       total = 0;
+      usleep(10000);
       while (keep_connection == TRUE)
          {
          vorbis_info_init(&vi);
@@ -860,11 +865,11 @@ void *codec_streamer_ogg(void *args)
          
          /* Build the packets */
          vorbis_analysis_headerout(&vd,&vc,&header_main,&header_comments,&header_codebooks);
+
          /* And prepare to stream them out */
          ogg_stream_packetin(&os,&header_main);
          ogg_stream_packetin(&os,&header_comments);
-         ogg_stream_packetin(&os,&header_codebooks);
- 
+         ogg_stream_packetin(&os,&header_codebooks); 
          while((result = ogg_stream_flush(&os, &og)))
             {
             if(!result) break;
@@ -886,9 +891,11 @@ void *codec_streamer_ogg(void *args)
             {
             if (!meta_f || newmeta == FALSE)
                {
-               usleep(10000);
-               if (!(frames = jack_ringbuffer_read_space(rightring) / sizeof (sample_t)))
+               if ((frames = jack_ringbuffer_read_space(rightring) / sizeof (sample_t)) < 2000)
+                  {
+                  usleep(10000);
                   continue;
+                  }
                frames = (frames > OGG_META_SRC_SIZE) ? OGG_META_SRC_SIZE : frames;
                buffer = vorbis_analysis_buffer(&vd, frames);
                if (channels == 2)
@@ -905,7 +912,7 @@ void *codec_streamer_ogg(void *args)
                   jack_ringbuffer_read(rightring, (char *)right_dm_buffer, frames * sizeof (sample_t));
                   bptr = buffer[0];
                   for (i = 0; i < frames ; i++)
-                     *bptr++ = (float)(*ldmptr++ + *rdmptr++) / 2.0F;
+                     *bptr++ = ((float)(*ldmptr++ + *rdmptr++)) * 0.5F;
                   }
                vorbis_analysis_wrote(&vd, frames);
 	       }
@@ -1002,9 +1009,11 @@ int stream_connect()
    if(!(encoder_pid=fork()))
       {
       char *samplerate;
-      char oggsr[20];
-      int bitrate = atoi(bandwidth);
-      char *lamepath = getenv("lame");
+      char *resamplerate;
+      int bitrate;
+      char *lamepath;
+      char *qualval;
+      int  highquality;
 
       close(writer[1]);			/* we will not be using these 2 */
       close(reader[0]);
@@ -1015,27 +1024,54 @@ int stream_connect()
       dup2(reader[1], STDOUT_FILENO);	/* Use stdin and stdout to access the pipe, now */
       close(reader[1]);			/* so we won't be needing this either now */
 
+      usleep(20000);
+      
+      bitrate = atoi(bandwidth);
+      lamepath = getenv("lame");
+      
       if (sr == 44100)			/* Only these two sample rates are supported */
+         {
          samplerate = "44.1";
+         resamplerate = "22.05";
+         }
       else
+         {
          samplerate = "48";
-      sprintf(oggsr, "%lu", sr); 
+         resamplerate = "24";
+         }
+      if (bitrate <= 24)
+         resamplerate = "16";
+      
+      highquality = high_quality != NULL && high_quality[0] == '1';
       
       if (!strcmp(format, "mp3"))
          {
+         usleep(10000);
          if (stereo_f)
             {
             if (bitrate >= 64)
-               execlp(lamepath, "lame", "-r", "-x", "-h", "-s", samplerate, "--cbr", "--noreplaygain", "-b", bandwidth, "-", NULL);
+               {
+               qualval = highquality ? "1" : "2";
+               execlp(lamepath, "lame", "-r", "-x", "-q", qualval, "-s", samplerate, "--cbr", "--noreplaygain", "-b", bandwidth, "-", NULL);
+               }
             else
-               execlp(lamepath, "lame", "-r", "-x", "-h", "-s", samplerate, "--cbr", "--noreplaygain", "-b", bandwidth, "--resample", "22.05", "-", NULL);
+               {
+               qualval = highquality ? "0" : "2";
+               execlp(lamepath, "lame", "-r", "-x", "-q", qualval, "-s", samplerate, "--cbr", "--noreplaygain", "-b", bandwidth, "--resample", resamplerate, "-", NULL);
+               }
 	    }
          else
 	    {
             if (bitrate >= 64)
-               execlp(lamepath, "lame", "-r", "-x", "-h", "-s", samplerate, "--cbr", "--noreplaygain", "-b", bandwidth, "-m", "s", "-a", "-", NULL);
+               {
+               qualval = highquality ? "1" : "2";
+               execlp(lamepath, "lame", "-r", "-x", "-q", qualval, "-s", samplerate, "--cbr", "--noreplaygain", "-b", bandwidth, "-m", "s", "-a", "-", NULL);
+               }
             else
-               execlp(lamepath, "lame", "-r", "-x", "-h", "-s", samplerate, "--cbr", "--noreplaygain", "-b", bandwidth, "-m", "s", "-a", "--resample", "22.05", "-", NULL);
+               {
+               qualval = highquality ? "0" : "2";
+               execlp(lamepath, "lame", "-r", "-x", "-q", qualval, "-s", samplerate, "--cbr", "--noreplaygain", "-b", bandwidth, "-m", "s", "-a", "--resample", resamplerate, "-", NULL);
+               }
 	    }
 	 fprintf(stderr, "Error: execlp failed to spawn lame process or bad option\n");
 	 }
@@ -1051,6 +1087,8 @@ int stream_connect()
       }
    signal(SIGCLD, SIG_IGN);
 
+   usleep(50000);
+      
    keep_connection = TRUE;	/* stops our read and write threads from exiting */
    keep_watchdog = TRUE;	/* stops our watchdog from exiting */
    if(pthread_create(&server_thread_h, NULL, server_thread, NULL))
@@ -1058,11 +1096,16 @@ int stream_connect()
       fprintf(stderr, "Failed to start server thread\n");
       goto error;
       }
+      
+   usleep(10000);
+      
    if (pthread_create(&codec_feeder_h, NULL, codec_feeder, NULL))
       {
       fprintf(stderr, "Failed to start codec feeder thread\n");
       goto error;
       }
+      
+   usleep(10000);
 
    return 0;
    error:
@@ -1083,11 +1126,13 @@ int stream_connect_ogg()
       strcpy(ai_channels, "1");		/* indicate stereo on bandwidths other than 32 */
    keep_connection = TRUE;	/* stops our read and write threads from exiting */
    keep_watchdog = TRUE;	/* stops our watchdog from exiting */
+   usleep(20000);
    if (pthread_create(&codec_streamer_ogg_h, NULL, codec_streamer_ogg, NULL))
       {
       fprintf(stderr, "Failed to start ogg codec/streamer thread\n");
       return 5;
       }
+   usleep(20000);
    return 0;
    }   
    
@@ -1122,6 +1167,7 @@ void *saver_thread(void *args)
    unsigned char *mp3buff;
    FILE *fp;
 
+   usleep(20000);
    if (!(mp3buff = (unsigned char *)malloc(MP3_BUF_SIZ)))
       {
       fprintf(stderr, "Unable to allocate space for mp3 buffer\n");
@@ -1142,12 +1188,12 @@ void *saver_thread(void *args)
 
    while (TRUE)
       {
+      usleep(50000);
       qty_read = read(sreader[0], mp3buff, MP3_BUF_SIZ);
       if (qty_read > 0)
          fwrite(mp3buff, 1, qty_read, fp);
       else
          break;
-      usleep(50000);
       }
    fclose (fp);
    close(sreader[0]);
@@ -1178,11 +1224,14 @@ int start_saving()
       perror("Failed to open pipe: ");
       return 5;
       }
-   
+   usleep(10000);
+      
    if(!(saver_pid=fork()))
       {
       char *samplerate;
+      char *resamplerate;
       char oggsr[20];
+      int bitrate;
       
       close(swriter[1]);		/* we will not be using these 2 */
       close(sreader[0]);
@@ -1193,21 +1242,39 @@ int start_saving()
       dup2(sreader[1], STDOUT_FILENO);	/* Use stdin and stdout to access the pipe, now */
       close(sreader[1]);			/* so we won't be needing this either now */
       
+      usleep(20000);
+      
       if (sr == 44100)			/* Only these two sample rates are supported */
+         {
          samplerate = "44.1";
+         resamplerate = "22.05";
+         }
       else
+         {
          samplerate = "48";
-      sprintf(oggsr, "%lu", sr); 
+         resamplerate = "24";
+         }
+      sprintf(oggsr, "%lu", sr);
+      bitrate = atoi(save_bitrate);
+      if (bitrate <= 32)
+         resamplerate = "16";
       
       if (!strcmp(save_format, "mp3"))
          {
          fprintf(stderr, "mp3 chosen\n");
-	 execlp(getenv("lame"), "lame", "-r", "-x", "-h", "-s", samplerate, "--cbr", "-b", save_bitrate, "--tt", save_title, "--ta", save_artist, "-", NULL);
-
+	 if (bitrate >= 64)
+            {
+            execlp(getenv("lame"), "lame", "-r", "-x", "-h", "-s", samplerate, "--cbr", "-b", save_bitrate, "--tt", save_title, "--ta", save_artist, "-", NULL);
+            }
+         else
+            {
+            execlp(getenv("lame"), "lame", "-r", "-x", "-h", "-s", samplerate, "--cbr", "-b", save_bitrate, "--tt", save_title, "--ta", save_artist, "--resample", resamplerate, "-", NULL);
+            }
 	 }
       else if (!strcmp(save_format, "ogg"))
          {
          fprintf(stderr, "ogg chosen\n");
+         usleep(10000);
 	 execlp(getenv("oggenc"), "oggenc", "-r", "-R", oggsr, "-q", save_bitrate, "-t", save_title, "-a", save_artist, "-", NULL);
 	 }
       else
@@ -1215,20 +1282,22 @@ int start_saving()
       exit(-5);
       }
    close(swriter[0]);		/* we are not using these 2 at this end */
-   close(sreader[1]);            
+   close(sreader[1]);
+   
    if (saver_pid == -1)
       {
       fprintf(stderr, "Failed to create child process for mp3/ogg encoding: ");
       goto error;
       }
    signal(SIGCLD, SIG_IGN);
-
+   usleep(30000);
    keep_saving = TRUE;	/* stops our read and write threads from exiting */
    if(pthread_create(&saver_thread_h, NULL, saver_thread, NULL))
       {
       fprintf(stderr, "Failed to start saver thread\n");
       goto error;
       }
+   usleep(30000);
    if (pthread_create(&save_codec_feeder_h, NULL, save_codec_feeder, NULL))
       {
       fprintf(stderr, "Failed to start save codec feeder thread\n");
@@ -1241,7 +1310,7 @@ int start_saving()
    close(sreader[0]);
    return 5;
    }
-    
+
 void interrupt_handler(int data)	/* eats the first ^C */
    {						/* the parent program will handle it */
    static int count = 0;
@@ -1259,10 +1328,16 @@ void interrupt_handler(int data)	/* eats the first ^C */
       }
    }
 
-int main(int argc, char *argv[])
+void my_jack_error_callback(const char *message)
+   {
+   printf("JACK: %s\n", message);
+   fflush(stdout);
+   }
+
+int main(int argc, char **argv)
    {
    FILE *fp = stdin;
-   char *client_name = 0;
+   char *client_name;
 
    if(sigfillset(&serversigset))
       {
@@ -1272,14 +1347,20 @@ int main(int argc, char *argv[])
    signal(SIGINT, interrupt_handler);
    
    srand(time(NULL));
-   client_name = (char *) malloc(9 * sizeof (char));
+   jack_set_error_function(my_jack_error_callback);
+   if(!(client_name = (char *) malloc(9 * sizeof (char))))
+      {
+      fprintf(stderr, "malloc failure\n");
+      return 1;
+      }
    strcpy(client_name, "idjc-srv");
    if((client = jack_client_new(client_name)) == 0)
       {
       fprintf(stderr, "jack server not running?\n");
       return 1;
       }
-   jack_set_process_callback (client, process, 0);
+   
+   jack_set_process_callback (client, process, NULL);
    jack_on_shutdown(client, shutdown, NULL);
    
    stream_left_port = jack_port_register(client, "lt_in", JACK_DEFAULT_AUDIO_TYPE, 
