@@ -30,6 +30,7 @@
 #include <vorbis/vorbisfile.h>
 #include <FLAC/all.h>
 #include <samplerate.h>
+#include <sndfile.h>
 #include "xlplayer.h"
 
 #define TRUE 1
@@ -56,6 +57,9 @@ static int get_playertype(struct xlplayer *self, OggVorbis_File *vf, FLAC__Strea
       ov_clear(vf);
    if (FLAC__metadata_get_streaminfo(self->pathname, metainfo))
       return XL_FLAC;
+   self->sf_info.format = 0;
+   if ((self->sndfile = sf_open(self->pathname, SFM_READ, &(self->sf_info))))
+      return XL_SNDFILE;
    return XL_XINE;
    }
 
@@ -72,7 +76,7 @@ static float *make_audio_to_float(struct xlplayer *self, float *buffer, uint8_t
    float fscale;
    const float half_randmax = (float)(RAND_MAX >> 1);
    float dscale;
- 
+
    msb_mask = 1UL << (bits_per_sample - 1);		/* negative number detector */
    neg_mask = (uint32_t)((~0UL) << (bits_per_sample));	/* negative number maker */
    fscale = 1.0F/(float)msb_mask;			/* multiplier to make the floating point range -1 to +1 */
@@ -95,12 +99,12 @@ static float *make_audio_to_float(struct xlplayer *self, float *buffer, uint8_t
             if (holder & msb_mask)
                holder |= neg_mask;
             if (self->dither && bits_per_sample < 20)
-               /* performs triangular dither no noise shaping to speak of */
+               /* adds triangular dither */
                *fptr++ = (((float)(int32_t)holder) * fscale) + 
                (((((float)rand_r(&self->seed)) - half_randmax) +
                (((float)rand_r(&self->seed)) - half_randmax)) * dscale); 
             else
-               *fptr++ = ((float)(int32_t)holder) * fscale;
+               *fptr++ = ((float)((int32_t)holder)) * fscale;
             }
          }
       }
@@ -131,12 +135,12 @@ static void demux_channel_data(struct xlplayer *self, sample_t *buffer, int num_
    int i;
    sample_t *lc, *rc, *src, gain;
    
-   if (!(self->leftbuffer = realloc(self->leftbuffer, self->op_buffersize)))
+   if ((!(self->leftbuffer = realloc(self->leftbuffer, self->op_buffersize))) && num_samples)
       {
       fprintf(stderr, "xlplayer: malloc failure");
       exit(5);
       }
-   if (!(self->rightbuffer = realloc(self->rightbuffer, self->op_buffersize)))
+   if ((!(self->rightbuffer = realloc(self->rightbuffer, self->op_buffersize))) && num_samples)
       {
       fprintf(stderr, "xlplayer: malloc failure");
       exit(5);
@@ -209,11 +213,14 @@ static void write_channel_data(struct xlplayer *self)
       }
    else
       {
-      jack_ringbuffer_write(self->left_ch, (char *)self->leftbuffer, self->op_buffersize);
-      jack_ringbuffer_write(self->right_ch, (char *)self->rightbuffer, self->op_buffersize);
-      samplecount = self->op_buffersize / sizeof (sample_t);
-      self->samples_written += samplecount;
-      self->sleep_samples += samplecount; 
+      if (self->op_buffersize)
+         {
+         jack_ringbuffer_write(self->left_ch, (char *)self->leftbuffer, self->op_buffersize);
+         jack_ringbuffer_write(self->right_ch, (char *)self->rightbuffer, self->op_buffersize);
+         samplecount = self->op_buffersize / sizeof (sample_t);
+         self->samples_written += samplecount;
+         self->sleep_samples += samplecount;
+         }
       self->write_deferred = FALSE;
       if (self->sleep_samples > 6000)
          {
@@ -319,7 +326,20 @@ static void flac_error_callback(const FLAC__FileDecoder *decoder,FLAC__StreamDec
    {
    struct xlplayer *self = client_data;
          
-   fprintf(stderr, "xlplayer: flac decoder detected an error in file %s\n", self->pathname);
+   switch (se)
+      {
+      case FLAC__STREAM_DECODER_ERROR_STATUS_LOST_SYNC:
+         fprintf(stderr, "xlplayer: %s: flac decoder error: lost sync\n%s\n", self->playername, self->pathname);
+         break;
+      case FLAC__STREAM_DECODER_ERROR_STATUS_BAD_HEADER:
+         fprintf(stderr, "xlplayer: %s: flac decoder error: bad header\n%s\n", self->playername, self->pathname);
+         break;
+      case FLAC__STREAM_DECODER_ERROR_STATUS_FRAME_CRC_MISMATCH:
+         fprintf(stderr, "xlplayer: %s: flac decoder error: frame crc mismatch\n%s\n", self->playername, self->pathname);
+         break;
+      default:
+         fprintf(stderr, "xlplayer: %s: flac decoder error: unknown error\n%s\n", self->playername, self->pathname);
+      }
    }
 
 static void *xlplayer_main(struct xlplayer *self)
@@ -328,13 +348,15 @@ static void *xlplayer_main(struct xlplayer *self)
    xine_stream_t       *stream = NULL;
    xine_audio_port_t   *ao_port = NULL;
    xine_audio_frame_t   frame;
-   char		       *configfile;
+   char		       *configfile = NULL;
    const char 	       *xinehomedir;
-            
+   const sf_count_t     sndfile_frameqty = 4096;
+   sf_count_t		sf_count;
+
    OggVorbis_File vf;
    vorbis_info *vi = NULL;
-   long pcmsamples;
-   uint8_t *ibuf = NULL;
+   long pcmsamples = 0, ov_retval;
+   char *ibuf = NULL;
    int bitstream = 0;
    
    FLAC__FileDecoder *decoder;		/* flac stuff */
@@ -348,15 +370,15 @@ static void *xlplayer_main(struct xlplayer *self)
 
    float      *flbuf = NULL;		/* generally used */
    int	       playertype = XL_BADFILE;
-   
+
    if ((xine = xine_new()))
       {
-      if (!(configfile = malloc(strlen(xinehomedir = xine_get_homedir()) + 14)))
+      if (!(configfile = malloc(strlen(xinehomedir = xine_get_homedir()) + 19)))
          {
          fprintf(stderr, "malloc failure\n");
          exit(5);
          }
-      sprintf(configfile, "%s%s", xinehomedir, "/.xine/config");
+      sprintf(configfile, "%s%s", xinehomedir, "/.idjc/xine-config");
       xine_config_load(xine, configfile);
       xine_init(xine);
       }
@@ -368,7 +390,7 @@ static void *xlplayer_main(struct xlplayer *self)
       fprintf(stderr, "xlplayer: %s could not initialise flac decoder\n", self->playername);
       }
 
-   for(;;)
+   for(self->up = TRUE; self->command != XL_THREADEXIT; self->watchdog_timer = 0)
       {
       switch (self->command)
          {
@@ -393,7 +415,7 @@ static void *xlplayer_main(struct xlplayer *self)
                self->command = XL_COMPLETE;
                }
             break;
-         case XL_EXIT:
+         case XL_CLEANUP:
             if (self->playlist)
                free(self->playlist);
             if (xine)
@@ -403,8 +425,8 @@ static void *xlplayer_main(struct xlplayer *self)
             if (self->flbuf)
                free(self->flbuf);
             free(configfile);
-            self->command = XL_COMPLETE;
-            return 0;				/* exit point */
+            self->command = XL_THREADEXIT;
+            continue;				/* exit point */
          }
       switch (self->playmode)
          {
@@ -453,7 +475,7 @@ static void *xlplayer_main(struct xlplayer *self)
                         fprintf(stderr, "xlplayer: %s src_new reports - %s\n", self->playername, src_strerror(src_error));
                         ov_clear(&vf);
                         self->playmode = XL_STOPPED;
-                        self->playmode = XL_COMPLETE;
+                        self->command = XL_COMPLETE;
                         continue;
                         }
                      src_data.output_frames = 0;
@@ -475,7 +497,7 @@ static void *xlplayer_main(struct xlplayer *self)
                         fprintf(stderr, "xlplayer: %s src_new reports - %s\n", self->playername, src_strerror(src_error));
                         ov_clear(&vf);
                         self->playmode = XL_STOPPED;
-                        self->playmode = XL_COMPLETE;
+                        self->command = XL_COMPLETE;
                         continue;
                         }
                      src_data.output_frames = 0;
@@ -512,6 +534,44 @@ static void *xlplayer_main(struct xlplayer *self)
                      self->suppress_flac_output = FALSE;
                      }
                   break;
+               case XL_SNDFILE:
+                  fprintf(stderr, "frames %ld\n", (long)self->sf_info.frames);
+                  fprintf(stderr, "samplerate %d\n", self->sf_info.samplerate);
+                  fprintf(stderr, "channels %d\n", self->sf_info.channels);
+                  fprintf(stderr, "format %d\n", self->sf_info.format);
+                  fprintf(stderr, "sections %d\n", self->sf_info.sections);
+                  fprintf(stderr, "seekable %d\n", self->sf_info.seekable);
+                  if (!(flbuf = realloc(flbuf, sizeof (float) * sndfile_frameqty * self->sf_info.channels)))
+                     {
+                     fprintf(stderr, "xlplayer: unable to allocate sndfile frames buffer\n");
+                     sf_close(self->sndfile);
+                     self->playmode = XL_STOPPED;
+                     self->command = XL_COMPLETE;
+                     continue;
+                     }
+                  if (self->sf_info.samplerate != self->samplerate)
+                     {
+                     fprintf(stderr, "xlplayer: %s configuring resampler\n", self->playername);
+                     src_state = src_new(SRC_LINEAR, self->sf_info.channels, &src_error);
+                     if (src_error)
+                        {
+                        fprintf(stderr, "xlplayer: %s src_new reports - %s\n", self->playername, src_strerror(src_error));
+                        sf_close(self->sndfile);
+                        self->playmode = XL_STOPPED;
+                        self->command = XL_COMPLETE;
+                        continue;
+                        }
+                     src_data.output_frames = 0;
+                     src_data.data_in = flbuf;
+                     src_data.data_out = NULL;
+                     src_data.src_ratio = (double)self->samplerate / (double)self->sf_info.samplerate;
+                     src_data.end_of_input = 0;
+                     resampling = TRUE;
+                     }
+                  else
+                     resampling = FALSE;
+                  sf_seek(self->sndfile, ((sf_count_t)self->seek_s) * ((sf_count_t)self->sf_info.samplerate), SEEK_SET);
+		  break;
                case XL_XINE:
                   if (!xine)
                      {
@@ -575,7 +635,7 @@ static void *xlplayer_main(struct xlplayer *self)
                         xine_dispose(stream);
                         xine_close_audio_driver(xine, ao_port);
                         self->playmode = XL_STOPPED;
-                        self->playmode = XL_COMPLETE;
+                        self->command = XL_COMPLETE;
                         continue;
                         }
                      src_data.output_frames = 0;
@@ -608,51 +668,76 @@ static void *xlplayer_main(struct xlplayer *self)
                   case XL_VORBIS:
                      if (resampling)
                         {
-                        ibuf = realloc(ibuf, sizeof (int16_t) * 6144 * vi->channels);
-                        pcmsamples = ov_read(&vf, ibuf, sizeof (int16_t) * 6144 * vi->channels, 0, 2, 1, &bitstream) / sizeof (int16_t) / vi->channels;
-                        if (pcmsamples < 0)
+                        ibuf = realloc(ibuf, 4096);
+                        ov_retval = ov_read(&vf, ibuf, 4096, 0, 2, 1, &bitstream);
+
+                        switch (ov_retval)
                            {
-                           fprintf(stderr, "xlplayer: %s ov_read reports an error on file '%s'\n", self->playername, self->pathname);
-                           continue;
-                           }
-                        src_data.input_frames = pcmsamples;
-                        src_data.end_of_input = pcmsamples == 0;
-                        src_data.data_in = realloc(src_data.data_in, src_data.input_frames * vi->channels * sizeof (float));
-                        src_data.output_frames = (int)(src_data.input_frames * src_data.src_ratio) + 2 + (512 * src_data.end_of_input);
-                        src_data.data_out = realloc(src_data.data_out, src_data.output_frames * vi->channels * sizeof (float));
-                        make_audio_to_float(self, src_data.data_in, ibuf, pcmsamples, 16, vi->channels);
-                        if ((src_error = src_process(src_state, &src_data)))
-                           {
-                           fprintf(stderr, "xlplayer: %s src_process reports - %s\n", self->playername, src_strerror(src_error));
-                           self->playmode = XL_EJECTING;
-                           continue;
-                           }
-                        demux_channel_data(self, src_data.data_out, src_data.output_frames_gen, vi->channels);
-                        write_channel_data(self);
-                        if (src_data.end_of_input)
-                           {
-                           self->playmode = XL_EJECTING;
-                           continue;
+                           case OV_HOLE:
+                              fprintf(stderr, "xlplayer: %s warning: hole in stream; probably harmless %s\n", self->playername, self->pathname);
+                              continue;
+                           case OV_EBADLINK:
+                              fprintf(stderr, "xlplayer: %s warning: invalid stream section or corrupt link %s\n", self->playername, self->pathname);
+                              continue;
+            		   default:
+               		      if (ov_retval < 0L)
+                                 {
+                                 fprintf(stderr, "xlplayer: %s warning: ov_read reports error code %ld\n", self->playername, pcmsamples);
+                                 self->playmode = XL_EJECTING;
+                                 continue;
+                                 }
+                              vi = ov_info(&vf, -1);
+                              pcmsamples = ov_retval / sizeof (int16_t) / vi->channels;
+                              src_data.input_frames = pcmsamples;
+                              src_data.end_of_input = pcmsamples == 0;
+                              src_data.data_in = realloc(src_data.data_in, src_data.input_frames * vi->channels * sizeof (float));
+                              src_data.output_frames = (int)(src_data.input_frames * src_data.src_ratio) + 2 + (512 * src_data.end_of_input);
+                              src_data.data_out = realloc(src_data.data_out, src_data.output_frames * vi->channels * sizeof (float));
+                              make_audio_to_float(self, src_data.data_in, (uint8_t *)ibuf, pcmsamples, 16, vi->channels);
+                              if ((src_error = src_process(src_state, &src_data)))
+                                 {
+                                 fprintf(stderr, "xlplayer: %s src_process reports - %s\n", self->playername, src_strerror(src_error));
+                                 self->playmode = XL_EJECTING;
+                                 continue;
+                                 }
+                              demux_channel_data(self, src_data.data_out, src_data.output_frames_gen, vi->channels);
+                              write_channel_data(self);
+                              if (src_data.end_of_input)
+                                 {
+                                 self->playmode = XL_EJECTING;
+                                 continue;
+                                 }
                            }
                         }
                      else
                         {
-                        ibuf = realloc(ibuf, sizeof (int16_t) * 6144 * vi->channels);
-                        pcmsamples = ov_read(&vf, ibuf, sizeof (int16_t) * 6144 * vi->channels, 0, 2, 1, &bitstream) / sizeof (int16_t) / vi->channels;
-                        if (pcmsamples < 0)
+                        ibuf = realloc(ibuf, 4096);
+                        ov_retval = ov_read(&vf, ibuf, 4096, 0, 2, 1, &bitstream);
+                        switch (ov_retval)
                            {
-                           fprintf(stderr, "xlplayer: %s ov_read reports an error on file '%s'\n", self->playername, self->pathname);
-                           continue;		/* skip this 'bad' frame */
+                           case OV_HOLE:
+                              fprintf(stderr, "xlplayer: %s warning: hole in stream; probably harmless %s\n", self->playername, self->pathname);
+                              continue;
+                           case OV_EBADLINK:
+                              fprintf(stderr, "xlplayer: %s warning: invalid stream section or corrupt link %s\n", self->playername, self->pathname);
+                              continue;
+                           case 0:
+                              self->playmode = XL_EJECTING;
+                              continue;
+            		   default:
+               		      if (ov_info < 0L)
+                                 {
+                                 fprintf(stderr, "xlplayer: %s warning: ov_read reports error code %ld\n", self->playername, pcmsamples);
+                                 self->playmode = XL_EJECTING;
+                                 continue;
+                                 }
+                              vi = ov_info(&vf, -1);
+                              pcmsamples = ov_retval / sizeof (uint16_t) / vi->channels;
+                              flbuf = realloc(flbuf, sizeof (float) * pcmsamples * vi->channels);
+                              flbuf = make_audio_to_float(self, flbuf, (uint8_t *)ibuf, pcmsamples, 16, vi->channels);
+                              demux_channel_data(self, flbuf, pcmsamples, vi->channels);
+                              write_channel_data(self);
                            }
-                        if (pcmsamples > 0)
-                           {
-                           flbuf = realloc(flbuf, sizeof (float) * pcmsamples * vi->channels);
-                           flbuf = make_audio_to_float(self, flbuf, ibuf, pcmsamples, 16, vi->channels);
-                           demux_channel_data(self, flbuf, pcmsamples, vi->channels);
-                           write_channel_data(self);
-                           }
-                        else
-                           self->playmode = XL_EJECTING;
                         }
                      break;
                   case XL_FLAC:
@@ -660,6 +745,31 @@ static void *xlplayer_main(struct xlplayer *self)
                      if (FLAC__file_decoder_get_state(decoder) != FLAC__FILE_DECODER_OK) 
                         self->playmode = XL_EJECTING;
                      break;
+                  case XL_SNDFILE:
+                     sf_count = sf_readf_float(self->sndfile, flbuf, sndfile_frameqty);
+                     if (resampling)
+                        {
+                        src_data.end_of_input = (sf_count == 0);
+                        src_data.input_frames = sf_count;
+                        src_data.output_frames = (int)(src_data.input_frames * src_data.src_ratio) + 2 + (512 * src_data.end_of_input);
+                        src_data.data_out = realloc(src_data.data_out, src_data.output_frames * self->sf_info.channels * sizeof (float));
+                        if ((src_error = src_process(src_state, &src_data)))
+                           {
+                           fprintf(stderr, "xlplayer: %s src_process reports - %s\n", self->playername, src_strerror(src_error));
+                           self->playmode = XL_EJECTING;
+                           continue;
+                           }
+                        demux_channel_data(self, src_data.data_out, src_data.output_frames_gen, self->sf_info.channels);
+                        }
+                     else
+                        demux_channel_data(self, flbuf, sf_count, self->sf_info.channels);
+                     write_channel_data(self);
+                     if (sf_count == 0)
+                        {
+                        self->playmode = XL_EJECTING;
+                        continue;
+                        }
+                     break;
                   case XL_XINE:
                      if (resampling)
                         {
@@ -703,7 +813,7 @@ static void *xlplayer_main(struct xlplayer *self)
             self->play_progress_ms = 0;
             if (resampling)
                {
-               if (src_data.data_in)
+               if (src_data.data_in && playertype != XL_SNDFILE)
                   free(src_data.data_in);
                if (src_data.data_out)
                   free(src_data.data_out);
@@ -717,6 +827,9 @@ static void *xlplayer_main(struct xlplayer *self)
                case XL_FLAC:
                   FLAC__file_decoder_finish(decoder);
                   break;
+               case XL_SNDFILE:
+                  sf_close(self->sndfile);
+                  break;
                case XL_XINE:
                   xine_close(stream);
                   xine_dispose(stream);
@@ -745,6 +858,8 @@ static void *xlplayer_main(struct xlplayer *self)
             break;
          } 
       }
+   self->command = XL_COMPLETE;
+   return 0;
    }
 
 struct xlplayer *xlplayer_create(int samplerate, size_t rbsize, char *playername, int *shutdown_f)
@@ -795,9 +910,13 @@ struct xlplayer *xlplayer_create(int samplerate, size_t rbsize, char *playername
    self->noflush = FALSE;
    self->suppress_flac_output = FALSE;
    self->jack_shutdown_f = shutdown_f;
+   self->watchdog_timer = 0;
    self->command = XL_COMPLETE;
    self->playmode = XL_STOPPED;
+   self->up = FALSE;
    pthread_create(&self->thread, NULL, (void *(*)(void *)) xlplayer_main, self);
+   while (self->up == FALSE)
+      usleep(10000);
    return self;
    }
  
@@ -805,7 +924,7 @@ void xlplayer_destroy(struct xlplayer *self)
    {
    if (self)
       {
-      self->command = XL_EXIT;
+      self->command = XL_CLEANUP;
       pthread_join(self->thread, NULL);
       jack_ringbuffer_free(self->left_ch);
       jack_ringbuffer_free(self->right_ch);
