@@ -23,12 +23,13 @@
 #include <stdlib.h>
 #include <samplerate.h>
 #include <FLAC/all.h>
+#include <sndfile.h>
+
+enum command_t {XL_COMPLETE, XL_PLAY, XL_EJECT, XL_CLEANUP, XL_THREADEXIT, XL_PLAYMANY};
 
-enum command_t {XL_COMPLETE, XL_PLAY, XL_EJECT, XL_EXIT, XL_PLAYMANY};
-   
 enum playmode_t {XL_STOPPED, XL_INITIATE, XL_PLAYING, XL_EJECTING };
 
-enum playertype_t {XL_XINE, XL_VORBIS, XL_FLAC, XL_BADFILE};
+enum playertype_t {XL_XINE, XL_VORBIS, XL_SNDFILE, XL_FLAC, XL_BADFILE};
 
 struct xlplayer
    {
@@ -77,6 +78,10 @@ struct xlplayer
    int noflush;				/* suppresses ringbuffer flushes */
    int suppress_flac_output;		/* used to suppress flac output so preventing odd seek noise */
    int *jack_shutdown_f;		/* inidcator that jack has shut down */
+   int watchdog_timer;			/* this will run away if the main loop has frozen */
+   SNDFILE *sndfile;			/* file handle from libsndfile for handling wav type files */
+   SF_INFO sf_info;			/* points to data about current libsndfile frame */
+   int up;				/* set to true when the player is fully initialised */
    };
 
 /* xlplayer_create: create an instance of the player */
