@@ -104,13 +104,13 @@ class IDJC_Media_Player:
          else:
             encoding = each
             rsmeta_name = u'<span foreground="gray">' + rich_safe(meta_name) + '</span>'
-            title_retval = rsmeta_name
+            title_retval = meta_name
             break
       else:
          encoding = "latin1"		# Have to default to something.
          meta_name = u'Unknown'
          rsmeta_name = u'<span foreground="gray">Unknown encoding</span>'
-         title_retval = rsmeta_name
+         title_retval = u"Unknown"
          
       filext = filext.lower()
       if filext == ".mp3" or filext == ".flac" or filext == ".wma" or filext == ".ogg":
@@ -268,6 +268,7 @@ class IDJC_Media_Player:
       
    # Shut down our media players when we exit.
    def cleanup(self):
+      self.exiting = True		# I added this because, well the reason totally sucks TBH
       if self.player_is_playing:
          self.stop.clicked()
       self.save_session()
@@ -425,9 +426,7 @@ class IDJC_Media_Player:
 	       self.player_is_playing = True
                print "Player has started"
 	    else:
-	       # Make our buttons match the player's state
-               self.player_is_playing = True
-	       self.next.clicked()
+               self.stop.clicked()
    
    def player_startup(self):
       if self.player_is_playing == True:
@@ -512,9 +511,10 @@ class IDJC_Media_Player:
          self.player_cid = -1
       if self.player_cid == -1:
          print "player startup was unsuccessful for file", self.music_filename
-         #return False
+         # we will treat this as successful and not return false so that end of track processing will take care of it
       print "player context id is %d\n" % self.player_cid
-      self.timeout_source_id = gobject.timeout_add(100, self.cb_play_progress_timeout, self.player_cid)
+      if self.player_cid % 2:
+         self.timeout_source_id = gobject.timeout_add(100, self.cb_play_progress_timeout, self.player_cid)
       return True
 
    def player_shutdown(self):
@@ -692,7 +692,7 @@ class IDJC_Media_Player:
    def update_time_stats(self):
       if self.pl_mode.get_active() != 0:		# optimisation -- this function uses a lot of cpu
          return
-      if self.is_playing:
+      if self.player_is_playing:
          tr = int(self.max_seek - self.progressadj.value)
          model = self.model_playing
          iter = model.iter_next(self.iter_playing)
@@ -771,11 +771,11 @@ class IDJC_Media_Player:
             print "termination due to end of track"
             self.invoke_end_of_track_policy()
             self.gapless = False
-            gtk.threads_leave()
+            gtk.gdk.threads_leave()
             return False
          if self.mixer_signal_f.value == False:
             self.silence_count += 1
-            if self.silence_count == 50:
+            if self.silence_count == 90:
                print "termination due to excessive silence"
                self.invoke_end_of_track_policy()
                gtk.gdk.threads_leave()
@@ -1064,22 +1064,25 @@ class IDJC_Media_Player:
 	 self.cb_progress(self.progressadj)
 
    def cb_event(self, widget, event, callback_data):
-      # This handles clicks to the progress bar
-      if callback_data == "ProgressPress":
-         self.progress_press = True
-	 if self.timeout_source_id:
-	    gobject.source_remove(self.timeout_source_id)
-         return False
-      elif callback_data == "ProgressRelease":
-         self.progress_press = False
-         if self.player_is_playing:
-            self.progress_current_figure = self.progressadj.get_value()
-            self.handle_motion_as_drop = True
-            gobject.idle_add(self.player_progress_value_changed_emitter)
-         return False
-      elif callback_data == "DigitalProgressPress":
-         self.digiprogress_click()
+      # Handle click to the play progress indicator
+      if callback_data == "DigitalProgressPress":
+         if event.button == 1:
+            self.digiprogress_click()
+         if event.button == 3:
+            self.parent.app_menu.popup(None, None, None, event.button, event.time)
          return True				# Prevent any focus therefore any cursor appearing
+      if event.button == 1:
+         # Handle click to the play progress bar
+         if callback_data == "ProgressPress":
+            self.progress_press = True
+            if self.timeout_source_id:
+               gobject.source_remove(self.timeout_source_id)
+         elif callback_data == "ProgressRelease":
+            self.progress_press = False
+            if self.player_is_playing:
+               self.progress_current_figure = self.progressadj.get_value()
+               self.handle_motion_as_drop = True
+               gobject.idle_add(self.player_progress_value_changed_emitter)
       return False
       
    # This is really a very convoluted workaround to achieve the effect of a connect_after
@@ -1312,17 +1315,17 @@ class IDJC_Media_Player:
 	             iter = model.get_iter(path)
                      gtk.gdk.threads_leave()
 	             if(position == gtk.TREE_VIEW_DROP_BEFORE or position == 							   gtk.TREE_VIEW_DROP_INTO_OR_BEFORE):
-                        gtk.threads_enter()
+                        gtk.gdk.threads_enter()
 	                iter = model.insert_before(iter, media_data)
-                        gtk.threads_leave()
+                        gtk.gdk.threads_leave()
 	             else:
-                        gtk.threads_enter()
+                        gtk.gdk.threads_enter()
 	                iter = model.insert_after(iter, media_data)
-                        gtk.threads_leave()
+                        gtk.gdk.threads_leave()
                   else:
-                     gtk.threads_enter()
+                     gtk.gdk.threads_enter()
                      iter = model.append(media_data)
-                     gtk.threads_leave()
+                     gtk.gdk.threads_leave()
 		  first = False
 	       else:
                   gtk.gdk.threads_enter()
@@ -1333,8 +1336,8 @@ class IDJC_Media_Player:
                gtk.gdk.threads_leave()
                gtk.gdk.threads_enter()
 	       gtk.main_iteration()
-               gtk.threads_leave()
-               gtk.threads_enter()
+               gtk.gdk.threads_leave()
+               gtk.gdk.threads_enter()
             gtk.gdk.threads_leave()
       self.reselect_please = True
       return False
@@ -1822,6 +1825,8 @@ class IDJC_Media_Player:
       return True
 	    
    def playtimerowconfig(self, tv_column, cell_renderer, model, iter):
+      if self.exiting:
+         return
       playtime = model.get_value(iter, 2)
       self.rowconfig(tv_column, cell_renderer, model, iter)
       if playtime == -11:
@@ -1835,6 +1840,8 @@ class IDJC_Media_Player:
          cell_renderer.set_property("text", text)
 
    def rowconfig(self, tv_column, cell_renderer, model, iter):
+      if self.exiting:
+         return
       celltext = model.get_value(iter, 0)
       if celltext[:4] == "<b>>":
          celltext = celltext[3:-4]
@@ -1897,7 +1904,7 @@ class IDJC_Media_Player:
          self.pl_statusbar.hide()
       
    def __init__(self, pbox, name, parent):
-      
+      self.exiting = False				# used to stop the running of problem code - don't ask
       # A box for the Stop/Start/Pause widgets
       self.hbox1 = gtk.HBox(True, 0)
       self.hbox1.set_border_width(2)
