@@ -110,7 +110,7 @@ def make_entry_line(parent, item, code):
 
 class mixprefs:
    def send_new_compressor_stats(self):
-      string_to_send = ":%.1f:%d:%d:%d:%d:%d:%d:%.1f:%d:%.1f:" % (
+      string_to_send = ":%.1f:%d:%d:%d:%d:%d:%d:%.1f:%d:%.1f:%d:" % (
       					self.ratio_adj.get_value(),
 					self.knee_adj.get_value(),
 					self.depth_adj.get_value(),
@@ -120,7 +120,8 @@ class mixprefs:
 					self.gain_relative.get_active(),
 					self.ducking_adj.get_value(),
 					self.ducking_hold_adj.get_value(),
-					self.de_ess_adj.get_value())
+					self.de_ess_adj.get_value(),
+                                        self.filter_slow.get_active())
       self.parent.mixer_write("COMP=%s\nACTN=compstats\nend\n" % string_to_send, True)
       
    def cb_compressor(self, widget, data):
@@ -187,52 +188,50 @@ class mixprefs:
       self.misc_session_frame.set_sensitive(state)
    
    def load_jack_port_settings(self):
-      home = os.environ.get("HOME")
-	 
-      if os.path.isfile(home + "/.idjc/micl"):
-         file = open(home + "/.idjc/micl", "r")
+      if os.path.isfile(self.parent.idjc + "micl"):
+         file = open(self.parent.idjc + "micl", "r")
 	 self.miclentry.set_text(file.readline()[:-1])
 	 self.miclcheck.set_active(file.readline() == "1\n")
 	 file.close()
 
-      if os.path.isfile(home + "/.idjc/micr"):
-         file = open(home + "/.idjc/micr", "r")
+      if os.path.isfile(self.parent.idjc + "micr"):
+         file = open(self.parent.idjc + "micr", "r")
 	 self.micrentry.set_text(file.readline()[:-1])
 	 self.micrcheck.set_active(file.readline() == "1\n")
 	 file.close()
 
-      if os.path.isfile(home + "/.idjc/audl"):
-         file = open(home + "/.idjc/audl", "r")
+      if os.path.isfile(self.parent.idjc + "audl"):
+         file = open(self.parent.idjc + "audl", "r")
 	 self.audlentry.set_text(file.readline()[:-1])
 	 self.audlcheck.set_active(file.readline() == "1\n")
 	 file.close()
       
-      if os.path.isfile(home + "/.idjc/audr"):
-         file = open(home + "/.idjc/audr", "r")
+      if os.path.isfile(self.parent.idjc + "audr"):
+         file = open(self.parent.idjc + "audr", "r")
 	 self.audrentry.set_text(file.readline()[:-1])
 	 self.audrcheck.set_active(file.readline() == "1\n")
 	 file.close()
 	 
-      if os.path.isfile(home + "/.idjc/strl"):
-         file = open(home + "/.idjc/strl", "r")
+      if os.path.isfile(self.parent.idjc + "strl"):
+         file = open(self.parent.idjc + "strl", "r")
 	 self.strlentry.set_text(file.readline()[:-1])
 	 self.strlcheck.set_active(file.readline() == "1\n")
 	 file.close()
       
-      if os.path.isfile(home + "/.idjc/strr"):
-         file = open(home + "/.idjc/strr", "r")
+      if os.path.isfile(self.parent.idjc + "strr"):
+         file = open(self.parent.idjc + "strr", "r")
 	 self.strrentry.set_text(file.readline()[:-1])
 	 self.strrcheck.set_active(file.readline() == "1\n")
 	 file.close()
       
-      if os.path.isfile(home + "/.idjc/auxl"):
-         file = open(home + "/.idjc/auxl", "r")
+      if os.path.isfile(self.parent.idjc + "auxl"):
+         file = open(self.parent.idjc + "auxl", "r")
 	 self.auxlentry.set_text(file.readline()[:-1])
 	 self.auxlcheck.set_active(file.readline() == "1\n")
 	 file.close()
 	       
-      if os.path.isfile(home + "/.idjc/auxr"):
-         file = open(home + "/.idjc/auxr", "r")
+      if os.path.isfile(self.parent.idjc + "auxr"):
+         file = open(self.parent.idjc + "auxr", "r")
 	 self.auxrentry.set_text(file.readline()[:-1])
 	 self.auxrcheck.set_active(file.readline() == "1\n")
 	 file.close()
@@ -241,8 +240,7 @@ class mixprefs:
       data.set_sensitive(not widget.get_active())
    
    def save_click(self, widget, data):
-      home = os.environ.get("HOME")
-      filename = home + "/.idjc/" + data[0].lower()
+      filename = self.parent.idjc + data[0].lower()
       file = open(filename, "w")
       if data[1].flags() & gtk.SENSITIVE:
          file.write(data[1].get_text() + "\n" + "0\n")
@@ -274,6 +272,17 @@ class mixprefs:
 	 file.close()
       except IOError:
          print "Error while writing out player defaults"
+      if self.ask_profile.get_active():
+         if os.path.isfile(self.parent.idjcroot + "do-not-ask-profile"):
+            try:
+               os.unlink(self.parent.idjcroot + "do-not-ask-profile")
+            except:
+               print "error removing file 'do-not-ask-profile'"
+      else:
+         try:
+            os.close(os.open(self.parent.idjcroot + "do-not-ask-profile", os.O_CREAT | os.O_WRONLY))
+         except:
+            print "error creating file 'do-not-ask-profile'"
 	 
    def load_player_prefs(self):
       try:
@@ -364,41 +373,11 @@ class mixprefs:
    def callback(self, widget, data):
       parent = self.parent
       if data == "basic streamer":
-         if parent.simplemixer == False:
-	    parent.simplemixer = True
-            parent.window.hide()
-            parent.player_right.stop.clicked()
-	    parent.jingles.window.hide()
-	    parent.crossadj.set_value(0)
-	    parent.crossadj.value_changed()
-	    parent.outercrossbox.hide()
-	    parent.volframe.hide()
-	    parent.vbox3R.hide()
-	    parent.jingles_button.hide()
-	    parent.mic_select_box.hide()
-	    parent.aux_select.hide()
-	    parent.meterbox.hide()
-            parent.frame2.hide()
-            parent.abox.hide()
-            parent.window.show()
-            parent.window.resize(parent.minwinx, parent.minwiny)
+         if parent.menu_feature_set.get_active():
+            parent.menu_feature_set.set_active(False)
       if data == "fully featured":
-         if parent.simplemixer == True:
-	    parent.simplemixer = False
-            parent.window.hide()
-	    parent.volframe.show()
-	    parent.vbox3R.show()
-	    parent.jingles_button.show()
-	    parent.aux_select.show()
-	    parent.mic_select_box.show()
-	    parent.outercrossbox.show()
-            parent.frame2.show()
-            parent.abox.show()
-            if self.mic_peak_toggle.get_active() or self.vu_toggle.get_active() or self.limiter_toggle.get_active() or self.stream_peak_toggle.get_active():
-	       self.parent.meterbox.show()
-	    parent.window.show()
-            parent.window.resize(parent.fullwinx, parent.fullwiny)
-	    parent.send_new_mixer_stats()
+         if parent.menu_feature_set.get_active() == False:
+            parent.menu_feature_set.set_active(True)
       if data == "enhanced-crossfader":
          if widget.get_active():
 	    parent.listen.show()
@@ -546,7 +525,7 @@ class mixprefs:
       hbox.pack_start(vbox, False, False, 9)     
       hbox.show()
       
-      # Sound meters selection
+      # Bits and Bobs
       
       frame = gtk.Frame(" " + bits_and_bobs_text + " ")
       frame.set_border_width(3)
@@ -555,12 +534,13 @@ class mixprefs:
       frame.add(hbox)
       frame.show()
       leftvbox = gtk.VBox()
+      midvbox = gtk.VBox()
       rightvbox = gtk.VBox()
-      rightmostvbox = gtk.VBox()
       hbox.add(leftvbox)
+      hbox.add(midvbox)
       hbox.add(rightvbox)
-      hbox.add(rightmostvbox)
       hbox.show()
+      # left column
       stream_peak_hbox = gtk.HBox()
       leftvbox.add(stream_peak_hbox)
       stream_peak_hbox.show()
@@ -570,54 +550,69 @@ class mixprefs:
       big_box_hbox = gtk.HBox()
       leftvbox.add(big_box_hbox)
       big_box_hbox.show()
-      dither_hbox = gtk.HBox()
-      rightmostvbox.add(dither_hbox)
-      dither_hbox.show()
-      leftvbox.show()
       mic_vu_hbox = gtk.HBox()
       rightvbox.add(mic_vu_hbox)
       mic_vu_hbox.show()
+      mp3_utf8_hbox = gtk.HBox()
+      leftvbox.add(mp3_utf8_hbox)
+      mp3_utf8_hbox.show()
+      leftvbox.show()
+      # mid column
       stream_vu_hbox = gtk.HBox()
-      rightvbox.add(stream_vu_hbox)
+      midvbox.add(stream_vu_hbox)
       stream_vu_hbox.show()
       normalize_hbox = gtk.HBox()
-      rightvbox.add(normalize_hbox)
+      midvbox.add(normalize_hbox)
       normalize_hbox.show()
-      rightvbox.show()
+      keeppass_box = gtk.HBox()
+      midvbox.add(keeppass_box)
+      keeppass_box.show()
+      ask_profile_hbox = gtk.HBox()
+      midvbox.add(ask_profile_hbox)
+      ask_profile_hbox.show()
+      midvbox.show()
+      # right column
       enhanced_cross_box = gtk.HBox()
-      rightmostvbox.add(enhanced_cross_box)
+      rightvbox.add(enhanced_cross_box)
       enhanced_cross_box.show()
       djalarmbox = gtk.HBox()
-      rightmostvbox.add(djalarmbox)
+      rightvbox.add(djalarmbox)
       djalarmbox.show()
       extm3ubox = gtk.HBox()
-      rightmostvbox.add(extm3ubox)
+      rightvbox.add(extm3ubox)
       extm3ubox.show()
-      rightmostvbox.show()
-      keeppass_box = gtk.HBox()
-      rightvbox.add(keeppass_box)
-      keeppass_box.show()
+      dither_hbox = gtk.HBox()
+      rightvbox.add(dither_hbox)
+      dither_hbox.show()
+      rightvbox.show()
 
-      self.stream_peak_toggle = gtk.CheckButton(stream_peak_toggle_text)
+      self.ask_profile = gtk.CheckButton(ask_profile_text)
+      ask_profile_hbox.pack_start(self.ask_profile, False, False, 0)
+      self.ask_profile.show()
+      self.stream_peak_toggle = gtk.CheckButton()
       stream_peak_hbox.pack_start(self.stream_peak_toggle, False, False, 5)
       self.stream_peak_toggle.connect("toggled", self.meter_callback, ("meter", self.parent.stream_peak_box))
       self.stream_peak_toggle.show()
-      self.mic_peak_toggle = gtk.CheckButton(mic_peak_toggle_text)
+      self.parent.str_pk_action.connect_proxy(self.stream_peak_toggle)
+      self.mic_peak_toggle = gtk.CheckButton()
       mic_peak_hbox.pack_start(self.mic_peak_toggle, False, False, 5)
       self.mic_peak_toggle.connect("toggled", self.meter_callback, ("meter", self.parent.mic_peak_box))
       self.mic_peak_toggle.show()
+      self.parent.mic_pk_action.connect_proxy(self.mic_peak_toggle) 
       self.bigger_box_toggle = gtk.CheckButton(big_box_toggle_text)
       big_box_hbox.pack_start(self.bigger_box_toggle, False, False, 5)
       self.bigger_box_toggle.connect("toggled", self.callback, "bigger box")
       self.bigger_box_toggle.show()
-      self.vu_toggle = gtk.CheckButton(show_vu_toggle_text)
+      self.vu_toggle = gtk.CheckButton()
       mic_vu_hbox.pack_start(self.vu_toggle, False, False, 0)
       self.vu_toggle.connect("toggled", self.meter_callback, ("meter", self.parent.vubox))
       self.vu_toggle.show()
-      self.limiter_toggle = gtk.CheckButton(comp_bar_toggle_text)
+      self.parent.show_vu_action.connect_proxy(self.vu_toggle)
+      self.limiter_toggle = gtk.CheckButton()
       stream_vu_hbox.pack_start(self.limiter_toggle, False, False, 0)
       self.limiter_toggle.connect("toggled", self.meter_callback, ("meter", self.parent.limiter_box))
       self.limiter_toggle.show()
+      self.parent.show_comp_action.connect_proxy(self.limiter_toggle)
       self.enhanced_crossfade = gtk.CheckButton("Enh. Crossfader")
       self.enhanced_crossfade.connect("toggled", self.callback, "enhanced-crossfader")
       #enhanced_cross_box.pack_start(self.enhanced_crossfade, False, False, 0)
@@ -644,6 +639,9 @@ class mixprefs:
       self.keeppass = gtk.CheckButton(keeppass_text)
       keeppass_box.pack_start(self.keeppass, False, False, 0)
       self.keeppass.show()
+      self.mp3_utf8 = gtk.CheckButton(mp3_utf8_text)
+      mp3_utf8_hbox.pack_start(self.mp3_utf8, False, False, 5)
+      self.mp3_utf8.show()
 
       outervbox.pack_start(frame, False, False, 0)
       
@@ -955,73 +953,66 @@ class mixprefs:
       mic_window = gtk.ScrolledWindow()
       mic_window.set_shadow_type(gtk.SHADOW_NONE)
       mic_window.set_border_width(8)
-      mic_window.set_policy(gtk.POLICY_NEVER, gtk.POLICY_ALWAYS)
+      mic_window.set_policy(gtk.POLICY_NEVER, gtk.POLICY_AUTOMATIC)
       viewport = gtk.Viewport()
       mic_window.add(viewport)
       mic_window.show()
-      #viewport.set_shadow_type(gtk.SHADOW_ETCHED_IN)
       panevbox = gtk.VBox()
       panevbox.set_border_width(3)
       viewport.add(panevbox)
       viewport.show()
+      
       compressorframe = gtk.Frame(" " + dnr_compressor_text + " ")
       panevbox.pack_start(compressorframe, False, False, 0)
       panevbox.show()
       compressorframe.set_border_width(3)
-      compressorvbox = gtk.VBox()
-      compressorvbox.set_border_width(4)
-      compressorvbox.set_spacing(5)
-      compressorframe.add(compressorvbox)
+      compressorhbox = gtk.HBox()
+      compressorframe.add(compressorhbox)
+      compressortable = gtk.Table(4, 3, True)
+      compressorhbox.pack_start(compressortable, True, True, 15)
+      compressorhbox.show()
+      compressortable.show()
       compressorframe.show()
-      ratio_knee_base_box = gtk.HBox(True)
-      compressorvbox.pack_start(ratio_knee_base_box, False, False, 0)
-      ratio_knee_base_box.show()
+      
+      compressortable.set_col_spacing(0, 12)
+      compressortable.set_col_spacing(1, 16)
+      
       ratio_box = gtk.HBox()
-      ratio_knee_base_box.pack_start(ratio_box, False, False, 0)
+      compressortable.attach(ratio_box, 0, 1, 0, 1, gtk.FILL | gtk.EXPAND, 0, 0, 0)
       ratio_box.show()
       knee_box = gtk.HBox()
-      ratio_knee_base_box.pack_start(knee_box, False, False, 0)
+      compressortable.attach(knee_box, 1, 2, 0, 1, gtk.FILL | gtk.EXPAND, 0, 0, 0)
       knee_box.show()
       depth_box = gtk.HBox()
-      ratio_knee_base_box.pack_start(depth_box, False, False, 0)
+      compressortable.attach(depth_box, 2, 3, 0, 1, gtk.FILL | gtk.EXPAND, 0, 0, 0)
       depth_box.show()
-      mid_box = gtk.HBox()
-      compressorvbox.pack_start(mid_box, False, False, 0)
-      mid_box.show()
-      left_mid_box = gtk.VBox()
-      left_mid_box.set_spacing(5)
-      mid_box.pack_start(left_mid_box, True, False, 0)
-      left_mid_box.show()
-      right_mid_box = gtk.VBox()
-      right_mid_box.set_spacing(5)
-      mid_box.pack_start(right_mid_box, True, False, 0)
-      right_mid_box.show()
+      
       attack_box = gtk.HBox()
-      left_mid_box.pack_start(attack_box, False, False, 0)
+      compressortable.attach(attack_box, 1, 2, 1, 2, gtk.FILL | gtk.EXPAND, 0, 0, 0)
       attack_box.show()
       release_box = gtk.HBox()
-      right_mid_box.pack_start(release_box, False, False, 0)
+      compressortable.attach(release_box, 1, 2, 2, 3, gtk.FILL | gtk.EXPAND, 0, 0, 0)
       release_box.show()
       gain_box = gtk.HBox()
-      left_mid_box.pack_start(gain_box)
+      compressortable.attach(gain_box, 2, 3, 1, 2, gtk.FILL | gtk.EXPAND, 0, 0, 0)
       gain_box.show()
       man_rel_box=gtk.VBox()
-      right_mid_box.pack_start(man_rel_box, False, False, 0)
+      compressortable.attach(man_rel_box, 2, 3, 2, 3, gtk.FILL | gtk.EXPAND, 0, 0, 0)
       man_rel_box.show()
-      comp_duck_deess_box = gtk.HBox(True)
-      compressorvbox.pack_start(comp_duck_deess_box, False, False, 0)
-      comp_duck_deess_box.show()
+      
       ducking_box = gtk.HBox()
-      comp_duck_deess_box.pack_start(ducking_box, False, False, 0)
+      compressortable.attach(ducking_box, 0, 1, 3, 4, gtk.FILL | gtk.EXPAND, 0, 0, 0)
       ducking_box.show()
       ducking_hold_box = gtk.HBox()
-      comp_duck_deess_box.pack_start(ducking_hold_box, False, False, 0)
+      compressortable.attach(ducking_hold_box, 1, 2, 3, 4, gtk.FILL | gtk.EXPAND, 0, 0, 0)
       ducking_hold_box.show()
       de_ess_box = gtk.HBox()
-      comp_duck_deess_box.pack_start(de_ess_box, False, False, 0)
+      compressortable.attach(de_ess_box, 2, 3, 3, 4, gtk.FILL | gtk.EXPAND, 0, 0, 0)
       de_ess_box.show()
-                  
-      sizegroup = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
+      rms_frame = gtk.Frame(" " + rms_text + " ")
+      rms_frame.set_border_width(4)
+      compressortable.attach(rms_frame, 0, 1, 1, 3, gtk.FILL | gtk.EXPAND, gtk.FILL | gtk.EXPAND, 0, 0)
+      rms_frame.show()
                   
       self.ratio_adj = gtk.Adjustment(3.0, 1.0, 20.0, 0.5, 0.5, 0.5) 
       self.ratio = gtk.SpinButton(self.ratio_adj, 0.0, 1)
@@ -1032,8 +1023,6 @@ class mixprefs:
       ratio_box.pack_start(label, False, False, 0)
       label.show()
       
-      sizegroup.add_widget(ratio_box)
-      
       self.knee_adj = gtk.Adjustment(-13.0, -99.0, 0.0, 1.0, 1.0, 1.0)
       self.knee = gtk.SpinButton(self.knee_adj, 0.5, 0)
       self.knee.connect("value-changed", self.cb_compressor, "knee")
@@ -1043,8 +1032,6 @@ class mixprefs:
       knee_box.pack_start(label, False, False, 0)
       label.show()
       
-      sizegroup.add_widget(knee_box)
-      
       self.depth_adj = gtk.Adjustment(27.0, 0.0, 99.0, 1.0, 1.0, 1.0)
       self.depth = gtk.SpinButton(self.depth_adj, 0.5, 0)
       self.depth.connect("value-changed", self.cb_compressor, "depth")
@@ -1054,13 +1041,8 @@ class mixprefs:
       depth_box.pack_start(label, False, False, 0)
       label.show()
 
-      sizegroup.add_widget(depth_box)
-
-      sizegroup2 = gtk.SizeGroup(gtk.SIZE_GROUP_HORIZONTAL)
-
       self.attack_adj = gtk.Adjustment(100.0, 1.0, 9999.0, 1.0, 1.0, 1.0)
       self.attack = gtk.SpinButton(self.attack_adj, 0.9, 0)
-      sizegroup2.add_widget(self.attack)
       self.attack.connect("value-changed", self.cb_compressor, "attack")
       attack_box.pack_start(self.attack, False, False, 0)
       self.attack.show()
@@ -1079,7 +1061,7 @@ class mixprefs:
       
       self.gain_adj = gtk.Adjustment(5.0, -60.0, 60.0, 1.0, 1.0, 1.0)
       self.gain = gtk.SpinButton(self.gain_adj, 0.5, 0)
-      sizegroup2.add_widget(self.gain)
+      #sizegroup2.add_widget(self.gain)
       self.gain.connect("value-changed", self.cb_compressor, "gain")
       gain_box.pack_start(self.gain, False, False, 0)
       self.gain.show()
@@ -1105,8 +1087,6 @@ class mixprefs:
       ducking_box.pack_start(label, False, False, 0)
       label.show()
       
-      sizegroup.add_widget(ducking_box)
-      
       self.ducking_hold_adj = gtk.Adjustment(30000.0, 0.0, 99999, 1000.0, 1000.0, 1000.0)
       self.ducking_hold = gtk.SpinButton(self.ducking_hold_adj, 0, 0)
       self.ducking_hold.connect("value-changed", self.cb_compressor, "duckhold")
@@ -1116,8 +1096,6 @@ class mixprefs:
       ducking_hold_box.pack_start(label, False, False, 0)
       label.show()
 
-      sizegroup.add_widget(ducking_hold_box)
-
       self.de_ess_adj = gtk.Adjustment(30.0, 0.0, 40.0, 0.5, 0.5, 0.5)
       self.de_ess = gtk.SpinButton(self.de_ess_adj, 0, 1)
       self.de_ess.connect("value-changed", self.cb_compressor, "de-ess")
@@ -1127,12 +1105,25 @@ class mixprefs:
       label.show()
       self.de_ess.show()
       
-      sizegroup.add_widget(de_ess_box)
-                  
+      rms_hbox = gtk.HBox()
+      rms_vbox = gtk.VBox()
+      rms_hbox.pack_start(rms_vbox, True, False, 0)
+      rms_frame.add(rms_hbox)
+      rms_hbox.show()
+      rms_vbox.show()
+      self.filter_fast = gtk.RadioButton(None, filter_fast_text)
+      self.filter_fast.connect("toggled", self.cb_compressor, "filterfast")
+      rms_vbox.add(self.filter_fast)
+      self.filter_fast.show()
+      self.filter_slow = gtk.RadioButton(self.filter_fast, filter_slow_text)
+      self.filter_slow.connect("toggled", self.cb_compressor, "filterslow")
+      rms_vbox.add(self.filter_slow)
+      self.filter_slow.show()
+      
       # Noise gate
       
       gatehbox = gtk.HBox()
-      gatehbox.set_border_width(4)
+      gatehbox.set_border_width(7)
       gateframe = gtk.Frame(" " + noise_gate_text + " ")
       gateframe.set_border_width(3)
       gateframe.add(gatehbox)
@@ -1141,12 +1132,15 @@ class mixprefs:
       gateframe.show()
 
       gateboxcol1 = gtk.VBox()
+      gateboxcol1.set_spacing(11)
       gatehbox.pack_start(gateboxcol1, True, False, 0)
       gateboxcol1.show()
       gateboxcol2 = gtk.VBox()
+      gateboxcol2.set_spacing(11)
       gatehbox.pack_start(gateboxcol2, True, False, 0)
       gateboxcol2.show()
       gateboxcol3 = gtk.VBox()
+      gateboxcol3.set_spacing(11)
       gatehbox.pack_start(gateboxcol3, True, False, 0)
       gateboxcol3.show()
             
@@ -1242,7 +1236,7 @@ class mixprefs:
       frame.show()
       
       filterbox = gtk.HBox()
-      filterbox.set_border_width(5)
+      filterbox.set_border_width(6)
       vbox.pack_start(filterbox, False, False, 0)
       filterbox.show()
 
@@ -1371,7 +1365,6 @@ class mixprefs:
       compressor_label = gtk.Label(microphone_tab_text)
       self.notebook.append_page(mic_window, compressor_label)
       compressor_label.show()
-      compressorvbox.show()
        
       # X-Chat IRC announcements tag.
       
@@ -1596,7 +1589,7 @@ class mixprefs:
                  
       jack_window = gtk.ScrolledWindow()
       jack_window.set_border_width(8)
-      jack_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_ALWAYS)
+      jack_window.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
       jack_vbox = gtk.VBox()
       jack_vbox.set_spacing(3)
       jack_vbox.set_border_width(4)
@@ -1670,7 +1663,6 @@ class mixprefs:
       jack_vbox.show()
       jacklabel.show()
       
-      
       frame = gtk.Frame()
       frame.set_border_width(9)
       vbox = gtk.VBox()
@@ -1684,8 +1676,8 @@ class mixprefs:
       vbox.pack_start(label, False, False, 0)
       label.show()
       
-      pixbuf = gtk.gdk.pixbuf_new_from_file(pkgdatadir + "icon" + gfext)
-      pixbuf = pixbuf.scale_simple(65, 70, gtk.gdk.INTERP_BILINEAR)
+      pixbuf = gtk.gdk.pixbuf_new_from_file(pkgdatadir + "logo" + gfext)
+      #pixbuf = pixbuf.scale_simple(265, 70, gtk.gdk.INTERP_BILINEAR)
       image = gtk.Image()
       image.set_from_pixbuf(pixbuf)
       vbox.pack_start(image, False, False, 8)
@@ -1747,9 +1739,9 @@ class mixprefs:
 	 "streampeak"	 : self.stream_peak_toggle,
 	 "limitergain"   : self.limiter_toggle,
 	 "comprelative"  : self.gain_relative,
+         "filterslow"    : self.filter_slow,
 	 "leftinvert"	 : self.left_invert,
 	 "rightinvert"	 : self.right_invert,
-	 #"enhancedcross" : self.enhanced_crossfade,
          "fadeout2"	 : self.fadeout,
 	 "announce_en"	 : self.announce_enable,
 	 "timer_en"	 : self.timer_enable,
@@ -1762,6 +1754,7 @@ class mixprefs:
          "normalize"	 : self.normalize,
          "dither"	 : self.dither,
          "recallsession" : self.restore_session_option,
+         "MP3usesUTF8"   : self.mp3_utf8,
          "keeppass"      : self.keeppass
 	 }
 	 
