@@ -73,8 +73,17 @@ class server_window:
                return True
    
    def servermodule_read(self, tag):
+      ioerrcount = 0
       while 1:
-         reply = self.serv_rply.readline()
+         try:
+            reply = self.serv_rply.readline()
+         except IOError:
+            print "servermodule_read: IOError detected"
+            ioerrcount += 1
+            if ioerrcount == 5:
+               self.parent.destroy_hard()
+            continue
+         ioerrcount = 0
          if len(reply) == 0:
             # the following function will do the relaunching duties
             self.servermodule_write("ACTN=status\nSTAG=%s\nend\n" % tag)
@@ -113,10 +122,13 @@ class server_window:
       if newmetatext != songname:
          artist = ""
          title = newmetatext
-      encodedmetatext = newmetatext.encode("latin1", "replace").strip()
+      if self.parent.prefs_window.mp3_utf8.get_active():
+         encodedmetatext = newmetatext.encode("utf8", "replace").strip()
+      else:
+         encodedmetatext = newmetatext.encode("latin1", "replace").strip()
       if len(encodedmetatext) > 512:
          encodedmetatext = encodedmetatext[:512]
-      self.servermodule_write("META=%s\nMART=%s\nMTIT=%s\nACTN=newmeta\nend\n" % (encodedmetatext,	artist, title))
+      self.servermodule_write("META=%s\nMART=%s\nMTIT=%s\nACTN=newmeta\nend\n" % (encodedmetatext, artist, title))
       print "updated metadata with:", newmetatext.encode(self.parent.denc, "replace").strip()
 
    def connect(self):
@@ -138,6 +150,7 @@ class server_window:
       self.stformat = self.format
       self.stbandwidth = self.bandwidth
       self.stabr_flag = self.abr_flag
+      self.quality = self.hq_checkbox.get_active()
       self.send_connect_command()
       self.update_button.set_sensitive(self.meta)
       
@@ -150,7 +163,7 @@ class server_window:
       				"connect", self.streamname, self.streamurl, self.streamdesc)
       cs += "SGEN=%s\nPUBF=%d\nMETF=%d\nMOUN=%s\n" % (
       				self.streamgenre, self.public, self.meta, self.mount)
-      cs += "ABRF=%d\nSTER=%d\nend\n" % (self.stabr_flag, self.stereo)
+      cs += "ABRF=%d\nSTER=%d\nQUAL=%d\nend\n" % (self.stabr_flag, self.stereo, self.quality)
       self.servermodule_write(cs)
       self.new_meta(self.parent.metadata, self.parent.artist, self.parent.title)
       self.last_message_time = 0
@@ -270,7 +283,7 @@ class server_window:
             if self.parent.prefs_window.timer_enable.get_active() and (self.last_message_time + (self.parent.prefs_window.intervaladj.get_value() * 60) < time.time()):
                self.last_message_time = time.time()
                try:
-	          file = open(self.parent.idjc + "timer.xchat", "w")
+	          file = open(self.parent.idjcroot + "timer.xchat", "w")
 	          fcntl.flock(file.fileno(), fcntl.LOCK_EX)
 	          file.write("d" + str(len(self.parent.prefs_window.timernickentry.get_text().encode("utf-8"))) + ":" + self.parent.prefs_window.timernickentry.get_text().encode("utf-8"))
 	          file.write("d" + str(len(self.parent.prefs_window.timerchannelsentry.get_text().encode("utf-8"))) + ":" + self.parent.prefs_window.timerchannelsentry.get_text().encode("utf-8"))
@@ -331,10 +344,12 @@ class server_window:
             self.mp3_stream_bitrate.show()
             self.ogg_stream_bitrate.hide()
             self.mp3_stream_bitrate.emit("changed")	# cause stereo box to be updated
+            self.hq_checkbox.set_sensitive(True)
          if self.format == "ogg":
             self.mp3_stream_bitrate.hide()
             self.ogg_stream_bitrate.show()
             self.ogg_stream_bitrate.emit("changed")
+            self.hq_checkbox.set_sensitive(False)
       if data == "mp3bitrate":
          self.stereo_checkbox.set_active(self.mp3_stereolist[widget.get_active()] == "1")
          self.bandwidth = widget.get_active_text()
@@ -394,6 +409,7 @@ class server_window:
          file.write("format=%d\n" % self.stream_format.get_active())
          file.write("mp3bitrate=%d\n" % self.mp3_stream_bitrate.get_active())
          file.write("oggbitrate=%d\n" % self.ogg_stream_bitrate.get_active())
+         file.write("highqualitymp3=%d\n" % self.hq_checkbox.get_active())
          file.write("mp3stereo=")	# saves state of stereo for each encoding
          for each in self.mp3_stereolist: file.write(each)
          file.write("\noggstereo=")
@@ -439,7 +455,11 @@ class server_window:
             if key == "format": self.stream_format.set_active(data == "1")
             if key == "mp3bitrate": self.mp3_stream_bitrate.set_active(int(data))
             if key == "oggbitrate": self.ogg_stream_bitrate.set_active(int(data))
+            if key == "highqualitymp3": self.hq_checkbox.set_active(int(data))
             if key == "mp3stereo":
+               while len(data) < len(self.mp3_stereolist):
+                  data += "0"
+                  print "extending mp3_stereolist"
                self.mp3_stereolist = list(each for each in data)
             if key == "oggstereo":
                self.ogg_stereolist = list(each for each in data)
@@ -709,11 +729,12 @@ class server_window:
             
       self.mp3_stream_bitrate = gtk.combo_box_new_text()
       self.mp3_stereolist = []
-      for each in ("320", "256", "192", "160", "128", "96", "64", "48", "32"):
+      for each in ("320", "256", "192", "160", "128", "96", "64", "48", "32", "24"):
          self.mp3_stream_bitrate.append_text(each)
          self.mp3_stereolist.append("1")
       self.mp3_stereolist[-1] = "0"
       self.mp3_stereolist[-2] = "0"
+      self.mp3_stereolist[-3] = "0"
       self.mp3_stream_bitrate.set_active(4)
       self.mp3_stream_bitrate.connect("changed", self.cb_encoding, "mp3bitrate")
       stream_bitrate_box.pack_start(self.mp3_stream_bitrate, True, False, 0)
@@ -743,6 +764,10 @@ class server_window:
       self.stereo_checkbox.connect("toggled", self.cb_encoding, "stereotoggle")
       stereo_checkbox_hbox.pack_start(self.stereo_checkbox, True, False, 0)
       self.stereo_checkbox.show()
+      self.hq_checkbox = gtk.CheckButton(high_quality_abbrev_text)
+      stereo_checkbox_hbox.pack_start(self.hq_checkbox, True, False, 0)
+      self.hq_checkbox.show()
+      
       enc_vbox.pack_start(stereo_checkbox_hbox, True, False, 8)
       stereo_checkbox_hbox.show()
       
