@@ -16,8 +16,8 @@
 #   Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
 
 appname= "Internet DJ Console"
-copyright = "Copyright 2006 Stephen Fairchild"
-copyright2 = u"\xA9 2006 Stephen Fairchild"	# As above but uses copyright character (c)
+copyright = "Copyright 2005-2006 Stephen Fairchild"
+copyright2 = u"\xA9 2005-2006 Stephen Fairchild"	# As above but uses copyright character (c)
 license = "Released under the GNU General Public License V2.0"
 # version is set in the configure.in file
 
@@ -288,6 +288,154 @@ class peakholdmeter(rawmeter):
       self.oldvalue = 0
       self.peakholditers = 3	# number of calls before the meter starts to decay
   
+# A dialog window for electing to create a new profile
+class profile_import_dialog(gtk.Dialog):
+   def __init__(self, title_text, additional_text, profiledir, newprofile):
+      gtk.Dialog.__init__(self, title_text, None, False, (gtk.STOCK_YES, gtk.RESPONSE_YES, gtk.STOCK_NO, gtk.RESPONSE_NO, ))
+      gtk.Dialog.set_icon_from_file(self, pkgdatadir + "icon" + gfext)
+      gtk.Dialog.set_resizable(self, False)
+      hbox = gtk.HBox(False, 20)
+      hbox.set_border_width(20)
+      self.vbox.pack_start(hbox, True, True, 0)
+      hbox.show()
+      image = gtk.Image()
+      image.set_from_stock(gtk.STOCK_DIALOG_QUESTION, gtk.ICON_SIZE_DIALOG)
+      hbox.pack_start(image, True, True, 0)
+      image.show()
+      vbox = gtk.VBox()
+      vbox.set_spacing(8)
+      hbox.pack_start(vbox, True, True, 0)
+      vbox.show()
+      if additional_text is not None:
+         if type(additional_text) is str:
+            additional_text = additional_text.splitlines()
+         for each in additional_text:
+            label = gtk.Label()
+            attrlist = pango.AttrList()
+            attrlist.insert(pango.AttrSize(12500, 0, len(each)))
+            label.set_attributes(attrlist)
+            label.set_text(each)
+            vbox.add(label)
+            label.show()
+      self.combobox = gtk.combo_box_new_text()
+      self.combobox.append_text("default")
+      for root, dirs, files in os.walk(profiledir):
+         for each in dirs:
+            if each != "default" and each != newprofile:
+               self.combobox.append_text(each)
+         break
+      self.combobox.set_active(0)
+      vbox.add(self.combobox)
+      self.combobox.show()
+
+# A dialog window for electing to create a new profile
+class select_profile_dialog(gtk.Dialog):
+   def profile_enter(self, widget):
+      if widget.get_text() != "":
+         self.enterbutton.clicked()
+      
+   def profile_changed(self, widget):
+      self.enterbutton.set_sensitive(len(widget.get_text()) != 0)
+   
+   def cb_keypress(self, widget, event):
+      # filter out undesired keys
+      if event.keyval == 65361: return False
+      if event.keyval == 65363: return False
+      if event.keyval == 65535: return False
+      if event.keyval == 65288: return False
+      if event.keyval == 65293: return False
+      if event.string.isalnum(): return False
+      return True
+   
+   def comboselection(self, widget):
+      if widget.get_active() == 0:
+         self.new_profile_box.show()
+         self.new_profile_box.grab_focus()
+         self.profile_changed(self.new_profile_box)
+      else:
+         self.new_profile_box.hide()
+         self.enterbutton.set_sensitive(True)
+
+   def __init__(self, title_text, additional_text, profiledir):
+      gtk.Dialog.__init__(self, title_text, None, False, None)
+      self.no_more_ask = gtk.CheckButton(no_more_ask_text)
+      self.action_area.pack_start(self.no_more_ask, False, False, 0)
+      self.no_more_ask.show()
+      self.enterbutton = gtk.Dialog.add_button(self, gtk.STOCK_OK, gtk.RESPONSE_OK)
+      gtk.Dialog.add_button(self, gtk.STOCK_QUIT, gtk.RESPONSE_CANCEL)
+      gtk.Dialog.set_icon_from_file(self, pkgdatadir + "icon" + gfext)
+      gtk.Dialog.set_resizable(self, False)
+      hbox = gtk.HBox(False, 20)
+      hbox.set_border_width(20)
+      self.vbox.pack_start(hbox, True, True, 0)
+      hbox.show()
+      image = gtk.Image()
+      image.set_from_stock(gtk.STOCK_DIALOG_QUESTION, gtk.ICON_SIZE_DIALOG)
+      hbox.pack_start(image, True, True, 0)
+      image.show()
+      vbox = gtk.VBox()
+      vbox.set_spacing(8)
+      hbox.pack_start(vbox, True, True, 0)
+      vbox.show()
+      if additional_text is not None:
+         if type(additional_text) is str:
+            additional_text = additional_text.splitlines()
+         for each in additional_text:
+            label = gtk.Label()
+            attrlist = pango.AttrList()
+            attrlist.insert(pango.AttrSize(12500, 0, len(each)))
+            label.set_attributes(attrlist)
+            label.set_text(each)
+            vbox.add(label)
+            label.show()
+      self.combobox = gtk.combo_box_new_text()
+      self.combobox.append_text(select_profile_new_text)
+      self.combobox.append_text("default")
+      for root, dirs, files in os.walk(profiledir):
+         for each in dirs:
+            if each != "default":
+               self.combobox.append_text(each)
+         break
+      self.combobox.set_active(1)
+      self.combobox.connect("changed", self.comboselection)
+      vbox.add(self.combobox)
+      self.combobox.show()
+      self.new_profile_box = gtk.Entry()
+      self.new_profile_box.connect("key_press_event", self.cb_keypress)
+      self.new_profile_box.connect("activate", self.profile_enter)
+      self.new_profile_box.connect("changed", self.profile_changed)
+      vbox.add(self.new_profile_box)
+ 
+# A dialog window for electing to create a new profile
+class new_profile_dialog(gtk.Dialog):
+   def __init__(self, title_text, additional_text):
+      gtk.Dialog.__init__(self, title_text, None, False, (gtk.STOCK_YES, gtk.RESPONSE_YES, gtk.STOCK_NO, gtk.RESPONSE_NO, gtk.STOCK_QUIT, gtk.RESPONSE_CANCEL))
+      gtk.Dialog.set_icon_from_file(self, pkgdatadir + "icon" + gfext)
+      gtk.Dialog.set_resizable(self, False)
+      hbox = gtk.HBox(False, 20)
+      hbox.set_border_width(20)
+      self.vbox.pack_start(hbox, True, True, 0)
+      hbox.show()
+      image = gtk.Image()
+      image.set_from_stock(gtk.STOCK_DIALOG_QUESTION, gtk.ICON_SIZE_DIALOG)
+      hbox.pack_start(image, True, True, 0)
+      image.show()
+      vbox = gtk.VBox()
+      vbox.set_spacing(8)
+      hbox.pack_start(vbox, True, True, 0)
+      vbox.show()
+      if additional_text is not None:
+         if type(additional_text) is str:
+            additional_text = additional_text.splitlines()
+         for each in additional_text:
+            label = gtk.Label()
+            attrlist = pango.AttrList()
+            attrlist.insert(pango.AttrSize(12500, 0, len(each)))
+            label.set_attributes(attrlist)
+            label.set_text(each)
+            vbox.add(label)
+            label.show()
+  
 # A dialog window to appear when shutdown is selected while still streaming.
 class idjc_shutdown_dialog:
    def window_attn(self, widget, event):
@@ -397,7 +545,10 @@ class MainWindow:
          self.metadata = self.songname			# now we need to send the new metadata
 	 print "song title: %s\n" % self.metadata.encode(self.denc, "replace")
 	 if self.metadata != u"":
-	    self.window.set_title("%s :: IDJC" % self.metadata)
+            if self.profile == "default":
+	       self.window.set_title("%s :: IDJC" % self.metadata)
+            else:
+               self.window.set_title("%s :: IDJC  (%s)" % (self.metadata, self.profile))
             tm = time.localtime()
             ts = "%02d:%02d :: " % (tm[3], tm[4])	# hours and minutes
             self.history_buffer.place_cursor(self.history_buffer.get_end_iter())
@@ -416,7 +567,7 @@ class MainWindow:
                file.close()
 
             try: # The song the DJ is listening to for the xchat /listening trigger
-               file = open(self.idjc + "songtitle", "w")
+               file = open(self.idjcroot + "songtitle", "w")
 	       file.write(self.metadata)		# <--- This is wrong
 	       file.close()
 	    except IOError:
@@ -433,12 +584,14 @@ class MainWindow:
 	       gobject.timeout_add(int(self.prefs_window.announcedelayadj.get_value() * 1000 - 500), self.announce_the_track, message_text)
             self.server_window.new_meta(self.metadata, self.artist, self.title)    
          else:
-	    self.window.set_title(self.appname)
-	 
-         
+            if self.profile == "default":
+	       self.window.set_title(self.appname)
+            else:
+               self.window.set_title(self.appname + "  (" + self.profile + ")")
+
    def announce_the_track(self, message):
       try:
-	 file = open(self.idjc + "announce.xchat", "w")
+	 file = open(self.idjcroot + "announce.xchat", "w")
 	 fcntl.flock(file.fileno(), fcntl.LOCK_EX)
          file.write(message)
 	 fcntl.flock(file.fileno(), fcntl.LOCK_UN)
@@ -456,6 +609,44 @@ class MainWindow:
 	 
    def callback(self, widget, data):
       print "%s was pressed" % data
+      if data == "Show about":
+         self.prefs_window.notebook.set_current_page(4)
+         self.prefs_window.window.present()
+      if data == "Features":
+         if widget.get_active():
+	    self.simplemixer = False
+            self.window.hide()
+	    self.volframe.show()
+	    self.vbox3R.show()
+	    self.jingles_button.show()
+	    self.aux_select.show()
+	    self.mic_select_box.show()
+	    self.outercrossbox.show()
+            self.frame2.show()
+            self.abox.show()
+            if self.prefs_window.mic_peak_toggle.get_active() or self.prefs_window.vu_toggle.get_active() or self.prefs_window.limiter_toggle.get_active() or self.prefs_window.stream_peak_toggle.get_active():
+	       self.meterbox.show()
+	    self.window.show()
+            self.window.resize(self.fullwinx, self.fullwiny)
+	    self.send_new_mixer_stats()
+         else:
+	    self.simplemixer = True
+            self.window.hide()
+            self.player_right.stop.clicked()
+	    self.jingles.window.hide()
+	    self.crossadj.set_value(0)
+	    self.crossadj.value_changed()
+	    self.outercrossbox.hide()
+	    self.volframe.hide()
+	    self.vbox3R.hide()
+	    self.jingles_button.hide()
+	    self.mic_select_box.hide()
+	    self.aux_select.hide()
+	    self.meterbox.hide()
+            self.frame2.hide()
+            self.abox.hide()
+            self.window.show()
+            self.window.resize(self.minwinx, self.minwiny)
       if data == "Advance":
          if self.crossfade.get_value() < 50:
             self.player_left.advance()
@@ -627,7 +818,7 @@ class MainWindow:
          fh.close()
       except:
          print "Error writing out tracks played data"
-      
+
    def restore_session(self):
       try:
          fh = open(self.session_filename, "r")
@@ -672,6 +863,9 @@ class MainWindow:
          print "Track history text is more than six hours old.  Disregarding.\n"
 
    def destroy_hard(self, widget=None, data=None):
+      if self.session_loaded:
+         self.save_session()
+         self.prefs_window.save_player_prefs()
       try:
          gtk.main_quit()
       except:
@@ -732,10 +926,11 @@ class MainWindow:
       self.destroy()
       return False
 
-   # the all singing all dancing error trapping mixer restarting mixer communications method
-   # all mixer write operations should go through here so that broken pipes can be trapped properly
-   def mixer_write(self, message, flush = False):
+   # all mixer write operations should go through here so that broken pipes can be handled properly
+   def mixer_write(self, message, flush = False, nowrite = False):
       try:
+         if nowrite:
+            raise IOError
          self.mixer_ctrl.write(message)
          if flush:
             self.mixer_ctrl.flush()
@@ -753,7 +948,8 @@ class MainWindow:
                   self.destroy_hard()
                self.last_chance = True
                print "mixer has crashed a second time"
-               mixer_write(self, message, flush)
+               self.mixer_write(message, flush, True)
+               return
             rply = self.mixer_read()
             if rply[:6] == "JACK: ":
                print rply
@@ -787,12 +983,19 @@ class MainWindow:
          self.jingles.stop.clicked()
          if self.jingles.interlude_player_track != "":
             self.jingles.start_interlude_player(self.jingles.interlude_player_track)
-         self.mixer_write(message, flush)	# resume what we were doing
+         if self.last_chance == False:
+            self.mixer_write(message, flush)		# resume what we were doing
       else:
          self.last_chance = False
 
-   def mixer_read(self):
-      line = self.mixer_rply.readline()
+   def mixer_read(self, iters = 0):
+      if iters == 5:
+         self.destroy_hard()
+      try:
+         line = self.mixer_rply.readline()
+      except IOError:
+         print "mixer_read: IOError detected"		# this can occur as a result of SIGUSR2 from the launcher
+         line = self.mixer_read(iters + 1)
       if line == "Segmentation Fault\n":
          line = ""
          print "Mixer reports a segmentation fault"
@@ -803,6 +1006,11 @@ class MainWindow:
    def vu_update(self, locking = True):
       if locking:
          gtk.gdk.threads_enter()
+         
+      if self.second_instance:
+         self.second_instance = False
+         self.window.present()
+         
       try:
          self.mixer_write("ACTN=requestlevels\nend\n", True)
       except ValueError, IOError:
@@ -874,15 +1082,28 @@ class MainWindow:
       else:
          self.minwinx.set_value(event.width)
          self.minwiny.set_value(event.height)
+         
+   def menu_activate(self, widget, event):
+      if event.type == gtk.gdk.BUTTON_PRESS and event.button == 3:
+         widget.popup(None, None, None, event.button, event.time)
+         return True
+      return False
+
+   # when a second instance of idjc is launched the program launcher will signal the running instance with SIGUSR2
+   def second_instance_handler(self, arg1, arg2):
+      print "the launch of a second instance of idjc was detected"
+      self.second_instance = True	# idjc will handle this event in good time
 
    class initfailed:
-      def __init__(self, errormessage = "Something bad happened and IDJC could not continue"):
+      def __init__(self, errormessage = "something bad happened and IDJC could not continue"):
          print errormessage
          
    class initcleanexit:
       pass
 
    def __init__(self, clargs):
+      self.second_instance = False
+      signal.signal(signal.SIGUSR2, self.second_instance_handler)
       signal.signal(signal.SIGINT, self.destroy_hard)
       
       self.appname = appname
@@ -891,9 +1112,33 @@ class MainWindow:
       self.license = license
       
       print appname, "Version", version, "\n" + copyright, "\n" + license + "\n"
+      arghelp = argver = False
+      self.profile = ""
       for each in clargs:
          if each == "-v" or each == "--version":
-            raise self.initcleanexit
+            argver = True
+         if each == "-h" or each == "--help" or each == "-hv" or each == "-vh":
+            arghelp = True
+      if argver == False and arghelp == False:
+         if len(clargs) == 1:
+            if clargs[0].isalnum():
+               self.profile = clargs[0]
+               print "using profile:", self.profile
+            else:
+               print "error: profile must be alphanumeric i.e. a-z A-Z 0-9"
+               raise self.initcleanexit
+         else:
+            if len(clargs) > 1:
+               print "non valid command line arguments: type idjc --help for more info"
+               raise self.initcleanexit
+            else:
+               print "no profile specified"
+      else:
+         if arghelp == True:
+            print "Usage: idjc [-v --version] [-h --help] [profilename]"
+            print "For more help visit http://idjc.sourceforge.net"
+         raise self.initcleanexit
+ 
       if shoutenabled == 0:
          print "Streaming is disabled.  To enable it install libshout then reinstall IDJC."
       self.denc = display_enc
@@ -902,23 +1147,134 @@ class MainWindow:
       home = os.environ.get("HOME")
       if home == "":
          raise self.initfailed("Error, HOME environment variable is not set")
+      
+      # Build the application data directory tree for idjc
       self.home = home
+      self.idjcroot = home + "/.idjc/"
       if not os.path.isdir(home + "/.idjc"):
          os.mkdir(home + "/.idjc")
       if not os.path.isdir(home + "/.idjc/jingles"):
          os.mkdir(home + "/.idjc/jingles")
-      self.idjc = home + "/.idjc/"
+      if not os.path.isdir(home + "/.idjc/profiles"):
+         os.mkdir(home + "/.idjc/profiles")
+      if not os.path.exists(home + "/.idjc/profiles/default"):
+         os.symlink(home + "/.idjc/", home + "/.idjc/profiles/default")
+      # Generate xine config file as needed
+      if not os.path.exists(home + "/.idjc/xine-config"):
+	 xconfig = None
+	 try:
+	    try:
+	       xconfig = open(home + "/.idjc/xine-config", "w")
+	       xconfig.write("# xine config file generated by idjc\n\n.version:2\n\naudio.driver:none\nvideo.driver:none\nengine.performance.memcpy_method:mmx\n")
+	    except:
+	       print "failed generation of xine-config file"
+	 finally:
+	    if xconfig:
+	       xconfig.close()
       
+      ask_profile = not os.path.isfile(self.idjcroot + "do-not-ask-profile")
+      if self.profile == "":
+         if ask_profile == False:
+            self.profile = "default"
+         else:
+            spd = select_profile_dialog(select_profile_title_text, select_profile_body_text, self.idjcroot + "profiles")
+            response = spd.run()
+            ask_profile = not spd.no_more_ask.get_active()
+            if response == gtk.RESPONSE_OK:
+               if spd.combobox.get_active() != 0:
+                  self.profile = spd.combobox.get_active_text()
+               else:
+                  self.profile = spd.new_profile_box.get_text()
+               spd.destroy()
+            else:
+               if response == gtk.RESPONSE_CANCEL:
+                  spd.destroy()
+               if ask_profile == False:
+                  try:
+                     os.close(os.open(self.parent.idjcroot + "do-not-ask-profile", os.O_CREAT | os.O_WRONLY))
+                  except:
+                     print "error creating file 'do-not-ask-profile'"
+               raise self.initcleanexit
+
+      if not os.path.exists(home + "/.idjc/profiles/" + self.profile):
+         npd = new_profile_dialog(new_profile_title_text, new_profile_body_text % self.profile)
+         response = npd.run()
+         if response != gtk.RESPONSE_DELETE_EVENT:
+            npd.destroy()
+         if response == gtk.RESPONSE_CANCEL:
+            print "user chose to quit"
+            raise self.initcleanexit
+         if response == gtk.RESPONSE_NO or response == gtk.RESPONSE_DELETE_EVENT:
+            print "user chose no -- we will continue with the default profile"
+            self.profile = "default"
+            self.idjc = home + "/.idjc/"
+         if response == gtk.RESPONSE_YES:
+            print "user chose to create a new profile"
+            self.idjc = home + "/.idjc/profiles/" + self.profile + "/"
+            os.mkdir(self.idjc[:-1])
+            os.mkdir(self.idjc + "jingles")
+            pid = profile_import_dialog(new_profile_title_text + ": " + self.profile, profile_import_body_text, self.idjcroot + "profiles", self.profile)
+            response = pid.run()
+            if response != gtk.RESPONSE_DELETE_EVENT:
+               if response == gtk.RESPONSE_YES:
+                  importprofile = pid.combobox.get_active_text()
+                  print "copying settings from %s profile" % importprofile
+                  os.system("cp %s %s" % (home + "/.idjc/profiles/" + importprofile + "/*", self.idjc))
+                  # make hard links of jingles files to save space
+                  os.system("cp -l %s %s" % (home + "/.idjc/profiles/" + importprofile + "/jingles/*", self.idjc + "jingles/"))
+               if response == gtk.RESPONSE_NO:
+                  print "no profile imported"
+               pid.destroy()
+      else:
+         self.idjc = home + "/.idjc/profiles/" + self.profile + "/"
+
+      while gtk.events_pending():	# update the GUI rendering to reflect the closure of the window
+         gtk.main_iteration()
+	 
+
+      self.session_loaded = False
       self.mixer_ctrl, self.mixer_rply = os.popen2([ libexecdir + "idjcmixer" ], 4096)
       if self.mixer_ctrl == 0 or self.mixer_rply == 0:
          print "Unable to open a pipe to the mixer"
 	 self.init_failed = True
          return
   
+      # check for a reply from the mixer.
+      self.last_chance = False
+      rply = jackrply = self.mixer_read()
+      while rply[:6] != "IDJC: ":
+         if rply == "":
+            print "mixer crashed"
+            message_dialog = gtk.MessageDialog(None, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_CLOSE, "The mixer module crashed during initialisation.\nThe most likely cause is a corrupt xine config file.\nDeleting the xine config files should solve the problem if relaunching IDJC should fail repeatedly.\ne.g. rm -fdr ~/.xine/")
+            message_dialog.set_icon_from_file(pkgdatadir + "icon" + gfext)
+            message_dialog.run()
+            message_dialog.destroy()
+            raise self.initfailed()
+         rply = self.mixer_read()
+         if rply[:6] == "JACK: ":
+            jackrply = rply
+      if rply[:17] == "IDJC: Sample rate":
+         self.samplerate = rply[18:].strip()
+         self.mixer_write("ACTN=sync\nend\n", True)
+         self.mixer_read()
+      else:
+         errormessage = "The JACK sound server needs to be running in order to run IDJC.\nIn order to manually start it try something like:\n\n		$ jackd -d alsa -r 44100 -p 2048\n\nIf you would like JACK to start automatically when you start IDJC try this:\n\n\techo \"/usr/bin/jackd -d alsa -r 44100\" >~/.jackdrc\n\n" + "If you have already done this it is possible another application or non-JACK sound server is using the sound card.\n\nPossible remedies would be to close the other audio app or configure the sound server to go into suspend mode after a brief amount of idle time."
+         if rply.strip() !="" and rply.strip() != "IDJC: Error":
+            errormessage = errormessage + "\nThe error message reported by JACK is:\n\n" + rply
+         message_dialog = gtk.MessageDialog(None, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_CLOSE, errormessage)
+         message_dialog.set_icon_from_file(pkgdatadir + "icon" + gfext)
+         message_dialog.run()
+         message_dialog.destroy()
+         raise self.initfailed()
+  
+      # create the GUI elements
       self.window_group = gtk.WindowGroup()
       self.window = gtk.Window(gtk.WINDOW_TOPLEVEL)
       self.window_group.add_window(self.window)
-      self.window.set_title(self.appname)
+      if self.profile == "default":
+         self.window.set_title(self.appname)
+      else:
+         self.window.set_title(self.appname + "  (" + self.profile + ")")
       self.window.set_border_width(8)
       self.window.connect("delete_event",self.delete_event)
       self.window.set_icon_from_file(pkgdatadir + "icon" + gfext)
@@ -1312,76 +1668,7 @@ class MainWindow:
       # We are done with vbox6 so lets show it
       self.vbox6.show()
       
-      # Pause here for a reply from the mixer.
-      self.last_chance = False
-      rply = jackrply = self.mixer_read()
-      while rply[:6] != "IDJC: ":
-         if rply == "":
-            print "mixer crashed"
-            message_dialog = gtk.MessageDialog(None, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_CLOSE, "The mixer module crashed during initialisation.\nThe most likely cause is a corrupt xine config file.\nDeleting the xine config files should solve the problem.\ne.g. rm -fdr ~/.xine/")
-            message_dialog.set_icon_from_file(pkgdatadir + "icon" + gfext)
-            message_dialog.run()
-            message_dialog.destroy()
-            raise self.initfailed()
-         rply = self.mixer_read()
-         if rply[:6] == "JACK: ":
-            jackrply = rply
-      if rply[:17] == "IDJC: Sample rate":
-         self.samplerate = rply[18:].strip()
-         self.mixer_write("ACTN=sync\nend\n", True)
-         self.mixer_read()
-      else:
-         if jackrply == "JACK: could not attach as client (duplicate client name?)\n":
-            message_dialog = gtk.MessageDialog(None, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_CLOSE, "At present only one instance of idjc can be running per user.\n\nIf idjc was running and it crashed it is possible that part of the program is still running.  Try the following in a console before attempting to run it again:\n\n   			$ killall idjcmixer idjcserver")
-         else:
-            errormessage = "The JACK sound server needs to be running in order to run IDJC.\nIn order to manually start it try something like:\n\n		$ jackd -d alsa -r 44100 -p 2048\n\nIf you would like JACK to start automatically when you start IDJC try this:\n\n\techo \"/usr/bin/jackd -d alsa -r 44100\" >~/.jackdrc\n\n" + "If you have already done this it is possible another application or non-JACK sound server is using the sound card.\n\nPossible remedies would be to close the other audio app or configure the sound server to go into suspend mode after a brief amount of idle time"
-            if rply.strip() !="" and rply.strip() != "IDJC: Error":
-               errormessage = errormessage + "\nThe error message reported by JACK is:\n\n" + rply
-            message_dialog = gtk.MessageDialog(None, 0, gtk.MESSAGE_INFO, gtk.BUTTONS_CLOSE, errormessage)
-         message_dialog.set_icon_from_file(pkgdatadir + "icon" + gfext)
-         message_dialog.run()
-         message_dialog.destroy()
-         raise self.initfailed()
-      
-      self.songname = u""
-      self.newmetadata = False
-      self.server_window = server_window(self)
-      self.server_window.load(self.idjc)
-      self.jingles = jingles(self)
-            
-      # Set variables
-      self.showing_left_file_requester = False
-      self.showing_right_file_requester = False
-      self.metadata = ""
-      self.simplemixer = False
-      self.crosspass = 0
-      self.alarm = False
-      self.NO_PHONE = 0
-      self.PUBLIC_PHONE = 1
-      self.PRIVATE_PHONE = 2
-      self.mixermode = self.NO_PHONE
-      self.jingles_playing = int_object(0)
-      self.interlude_playing = int_object(0)
-      self.player_left.playtime_elapsed = int_object(0)
-      self.player_right.playtime_elapsed = int_object(0)
-      self.player_left.mixer_playing = int_object(0)
-      self.player_right.mixer_playing = int_object(0)
-      self.player_left.mixer_signal_f = int_object(0)
-      self.player_right.mixer_signal_f = int_object(0)
-      self.player_left.mixer_cid = int_object(0)
-      self.player_right.mixer_cid = int_object(0)
-      self.jingles.mixer_jingles_cid = int_object(0)
-      self.jingles.mixer_interlude_cid = int_object(0)
-      self.player_left.runout = int_object(0)
-      self.player_right.runout = int_object(0)
-      self.fullwinx = int_object(1)
-      self.fullwiny = int_object(1)
-      self.minwinx = int_object(1)
-      self.minwiny = int_object(1)
-      self.in_vu_timeout = False
-      self.vucounter = 0
-      self.session_filename = self.idjc + "main_session"
-
+      # The various meters
       self.meterbox = gtk.HBox()
       self.hbox9.pack_start(self.meterbox, False, False, 0)
       self.meterbox.show()
@@ -1443,9 +1730,162 @@ class MainWindow:
       hbox.show()
       self.meterbox.pack_start(self.limiter_box, False, False, 0)
       self.limiter_box.show()
-                  
-      # Originally designed to get the stats for the meters from the mixer, it now pulls
-      # out much more info than that now.
+
+      # Main window context menu
+      self.app_menu = gtk.Menu()
+      self.app_menu.show()
+      self.window.connect_object("event", self.menu_activate, self.app_menu)
+      
+      aboutimg = gtk.Image()
+      aboutimg.set_from_stock(gtk.STOCK_ABOUT, gtk.ICON_SIZE_MENU)
+      aboutimg.show()
+      aboutlabel = gtk.Label("About")
+      aboutlabel.show()
+      aboutbox = gtk.HBox()
+      aboutbox.show()
+      aboutbox.pack_start(aboutimg, False, False, 0)
+      aboutbox.pack_start(aboutlabel, False, False, 0)
+      menu_about = gtk.MenuItem(None, False)
+      menu_about.connect("activate", self.callback, "Show about")
+      menu_about.show()
+      menu_about.add(aboutbox)
+      self.app_menu.append(menu_about)
+      
+      sep = gtk.SeparatorMenuItem()
+      sep.show()
+      self.app_menu.append(sep)
+      
+      viewimg = gtk.Image()
+      viewimg.show()
+      viewlabel = gtk.Label("View")
+      viewlabel.show()
+      viewbox = gtk.HBox()
+      viewbox.show()
+      viewbox.pack_start(viewimg, False, False, 8)
+      viewbox.pack_start(viewlabel, False, False, 0)
+      menu_view = gtk.MenuItem(None, False)
+      menu_view.show()
+      menu_view.add(viewbox)
+      self.app_menu.append(menu_view)
+     
+      sep = gtk.SeparatorMenuItem()
+      sep.show()
+      self.app_menu.append(sep)
+      
+      quitimg = gtk.Image()
+      quitimg.set_from_stock(gtk.STOCK_QUIT, gtk.ICON_SIZE_MENU)
+      quitimg.show()
+      quitlabel = gtk.Label("Quit")
+      quitlabel.show()
+      quitbox = gtk.HBox()
+      quitbox.show()
+      quitbox.pack_start(quitimg, False, False, 0)
+      quitbox.pack_start(quitlabel, False, False, 0)
+      menu_quit = gtk.MenuItem(None, False)
+      menu_quit.connect_object("activate", self.delete_event, self.window, None)
+      menu_quit.show()
+      menu_quit.add(quitbox)
+      self.app_menu.append(menu_quit)
+
+      # The view submenu of app_menu
+      view_menu = gtk.Menu()
+      view_menu.show()
+      menu_view.set_submenu(view_menu)
+      
+      menu_str_pk = gtk.CheckMenuItem()
+      menu_str_pk.show()
+      view_menu.append(menu_str_pk)
+      self.str_pk_action = gtk.ToggleAction(None, stream_peak_toggle_text, None, None)
+      self.str_pk_action.connect_proxy(menu_str_pk)
+      
+      menu_mic_pk = gtk.CheckMenuItem()
+      menu_mic_pk.show()
+      view_menu.append(menu_mic_pk)
+      self.mic_pk_action = gtk.ToggleAction(None, mic_peak_toggle_text, None, None)
+      self.mic_pk_action.connect_proxy(menu_mic_pk)
+
+      menu_show_vu = gtk.CheckMenuItem()
+      menu_show_vu.show()
+      view_menu.append(menu_show_vu)
+      self.show_vu_action = gtk.ToggleAction(None, show_vu_toggle_text, None, None)
+      self.show_vu_action.connect_proxy(menu_show_vu)
+      
+      menu_show_comp = gtk.CheckMenuItem()
+      menu_show_comp.show()
+      view_menu.append(menu_show_comp)
+      self.show_comp_action = gtk.ToggleAction(None, comp_bar_toggle_text, None, None)
+      self.show_comp_action.connect_proxy(menu_show_comp)
+      
+      sep = gtk.SeparatorMenuItem()
+      sep.show()
+      view_menu.append(sep)
+      
+      self.menu_feature_set = gtk.CheckMenuItem(fully_featured_text)
+      self.menu_feature_set.set_active(True)
+      self.menu_feature_set.connect("activate", self.callback, "Features")
+      self.menu_feature_set.show()
+      view_menu.append(self.menu_feature_set)
+      
+      sep = gtk.SeparatorMenuItem()
+      sep.show()
+      view_menu.append(sep)
+      
+      menu_prefs = gtk.MenuItem(prefs_button_text)
+      menu_prefs.connect("activate", self.callback, "Prefs")
+      menu_prefs.show()
+      view_menu.append(menu_prefs)
+      
+      menu_server = gtk.MenuItem(server_button_text)
+      menu_server.connect("activate", self.callback, "Server")
+      menu_server.show()
+      view_menu.append(menu_server)
+      
+      menu_jingles = gtk.MenuItem(jingles_button_text)
+      menu_jingles.connect("activate", self.callback, "Jingles")
+      menu_jingles.show()
+      view_menu.append(menu_jingles)
+
+      # Instantiate the other windows
+      self.songname = u""
+      self.newmetadata = False
+      self.server_window = server_window(self)
+      self.server_window.load(self.idjc)
+      self.jingles = jingles(self)
+            
+      # Variable initialisation
+      self.showing_left_file_requester = False
+      self.showing_right_file_requester = False
+      self.metadata = ""
+      self.simplemixer = False
+      self.crosspass = 0
+      self.alarm = False
+      self.NO_PHONE = 0
+      self.PUBLIC_PHONE = 1
+      self.PRIVATE_PHONE = 2
+      self.mixermode = self.NO_PHONE
+      self.jingles_playing = int_object(0)
+      self.interlude_playing = int_object(0)
+      self.player_left.playtime_elapsed = int_object(0)
+      self.player_right.playtime_elapsed = int_object(0)
+      self.player_left.mixer_playing = int_object(0)
+      self.player_right.mixer_playing = int_object(0)
+      self.player_left.mixer_signal_f = int_object(0)
+      self.player_right.mixer_signal_f = int_object(0)
+      self.player_left.mixer_cid = int_object(0)
+      self.player_right.mixer_cid = int_object(0)
+      self.jingles.mixer_jingles_cid = int_object(0)
+      self.jingles.mixer_interlude_cid = int_object(0)
+      self.player_left.runout = int_object(0)
+      self.player_right.runout = int_object(0)
+      self.fullwinx = int_object(1)
+      self.fullwiny = int_object(1)
+      self.minwinx = int_object(1)
+      self.minwiny = int_object(1)
+      self.in_vu_timeout = False
+      self.vucounter = 0
+      self.session_filename = self.idjc + "main_session"
+
+      # Variable map for stuff read from the mixer
       self.vumap = {
          "mic_l_peak"	: self.mic_l_peak,
 	 "mic_r_peak"	: self.mic_r_peak,
@@ -1476,6 +1916,7 @@ class MainWindow:
       self.prefs_window = mixprefs(self)
       self.prefs_window.load_player_prefs()
       self.prefs_window.apply_player_prefs()
+      self.prefs_window.ask_profile.set_active(ask_profile)
       self.vutimeout = gobject.timeout_add(50, self.vu_update)
       self.statstimeout = gobject.timeout_add(500, self.stats_update)
 
@@ -1492,7 +1933,8 @@ class MainWindow:
          self.player_left.restore_session()
          self.player_right.restore_session()
          self.restore_session()
- 
+      self.session_loaded = True
+       
       self.window.show()
    
    def main(self):
@@ -1537,6 +1979,8 @@ if __name__ == "__main__":
          retcode = 5
       except MainWindow.initcleanexit:
          pass
+      except KeyboardInterrupt:
+         pass
       else:
          try:
             run_instance.main()
