@@ -102,6 +102,7 @@ normfall_text = ("Fall", "Fall", "Fall")[li]
 normalizer_toggle_text = ("Str. Normalize", "Str. Normalize", "Str. Normalize")[li]
 normdefaults_text = ("Defaults", "Standardeinstellungen", "Defaults")[li]
 # these correspond to str_peak_text and friends
+ask_profile_text =  ("Ask For Profile", "Ask For Profile", "Ask For Profile")[li]
 stream_peak_toggle_text = ("Stream Peak", "Stream Spitze", "Stream Peak")[li]
 mic_peak_toggle_text = ("Mic Peak", "Mik Spitze", "Mic Peak")[li]
 show_vu_toggle_text = ("Show VU", "Zeige VU", "Show VU")[li]
@@ -116,6 +117,7 @@ player_2_text = ("Player 2", "Player 2", "Player 2")[li]
 count_up_text = ("Count Up", "Aufsteigend", "Count Up")[li]
 count_down_text = ("Count Down", "Absteigend", "Count Down")[li]
 restore_session_text = ("Restore previous session", "Einstellungen merken", "Restore previous session")[li]
+mp3_utf8_text = ("MP3meta=utf-8", "MP3meta=utf-8", "MP3meta=utf-8")[li]
 
 # Stuff in the Microphone tab of prefs window
 dnr_compressor_text = ("Dynamic Range Compressor", "Dynamic Range Kompressor", "Dynamic Range Compressor")[li]
@@ -123,7 +125,7 @@ comp_ratio_text = ("Ratio", "Ratio", "Ratio")[li]
 comp_knee_text = ("Knee", "Knee", "Knee")[li]
 comp_depth_text = ("Depth", "Depth", "Depth")[li]
 comp_attack_text = ("Attack", "Attack", "Attack")[li]
-comp_release_text = ("Release", "Release", "Release")[li]
+comp_release_text = ("Decay", "Decay", "Decay")[li]
 comp_gain_text = ("Gain", "Gain", "Gain")[li]
 comp_relative_text = ("Relative", "Relativ", "Relative")[li]
 comp_manual_text = ("Manual", "Manuell", "Manual")[li]
@@ -137,6 +139,9 @@ ng_delay_text = ("Delay", "Delay", "Delay")[li]
 attenuation_text = ("Atten.", "Attenuation", "Atten.")[li]
 rise_text = ("Rise", "Rise", "Rise")[li]
 fall_text = ("Fall", "Fall", "Fall")[li]
+rms_text = ("Filter", "Filter", "Filter")[li]
+filter_fast_text = ("3ms RMS", "3ms RMS", "3ms RMS")[li]
+filter_slow_text = ("30ms RMS", "30ms RMS", "30ms RMS")[li]
 af_h_sm_text = ("Audio Filter, Headroom, and Stereo Mix","Audio Filter, Headroom, und Stereo Mix", "Audio Filter, Headroom, and Stereo Mix")[li]
 bass_cut_text = ("Bass cut:", "Bass cut:", "Bass cut:")[li]
 off_text = ("Off", "Aus", "Off")[li]
@@ -287,6 +292,8 @@ bitrate_text = ("Bitrate", "Bitrate", "Bitrate")[li]
 
 stereo_text = ("Stereo", "Stereo", "Stereo")[li]
 
+high_quality_abbrev_text = ("HQ", "HQ", "HQ")[li]
+
 # Fixes for folks that like SuSE for some strange reason.
 
 big_box_toggle_text = ("Counter fix", "Counter fix", "Counter fix")[li]
@@ -309,5 +316,17 @@ remaining_text = ("Remaining", "Verbleibend", "Remaining")[li]
 finish_text = ("Finish", "Ende", "Finish")[li]
 block_size_text = ("Block size", "Blockgröße", "Block size")[li]
 
+# Profile creation dialogs
+
+new_profile_title_text = ("IDJC - New Profile", "IDJC - New Profile", "IDJC - New Profile")[li] 
+new_profile_body_text = ("Profile '%s' does\nnot currently exist.\nWould you like to create it?", "Profile '%s' does\nnot currently exist.\nWould you like to create it?", "Profile '%s' does\nnot currently exist.\nWould you like to create it?")[li]
+profile_import_body_text = ("You can choose to import settings from\none of the existing profiles\nlisted in the drop down box below.", "You can choose to import settings from\none of the existing profiles\nlisted in the drop down box below.", "You can choose to import settings from\none of the existing profiles\nlisted in the drop down box below.")[li]
+
+# Profile selection dialog
+
+select_profile_title_text = ("IDJC - Profile Chooser", "IDJC - Profile Chooser", "IDJC - Profile Chooser")[li]
+select_profile_body_text = ("Which profile do you wish to use?", "Which profile do you wish to use?", "Which profile do you wish to use?")[li]
+select_profile_new_text = ("A New Profile", "A New Profile", "A New Profile")[li]
+no_more_ask_text = ("Don't ask again", "Don't ask again", "Don't ask again")[li]
 
 del li
